# -*- coding:utf-8 -*-

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from inmanage_sdk.util import RegularCheckUtil
from inmanage_sdk.command import IpmiFunc
import sys
import os
import re
import time
from inmanage_sdk.interface.IBase import IBase
from inmanage_sdk.interface.ResEntity import (ResultBean, CapabilitiesBean, Sensor, SensorBean, Temperature,
                                     TemperatureBean, Voltage, VoltBean,)
import collections

rootpath = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.append(os.path.join(rootpath, "command"))
sys.path.append(os.path.join(rootpath, "util"))


class Base(IBase):

    # 100+为utool定义
    # 1-9为ipmitool工具定义

    ERR_dict = {
        1: 'Ipmi information get error',
        2: 'Ipmi parameter is null',
        3: 'Ipmi parameter error',
        4: 'Cannot create ipmi link, please check host/username/pasword',
        5: 'Cannot connect to server, please check host/username/pasword',
        6: 'Memory exception',
        7: 'Cannot connect to the server, please check host/username/pasword',
        8: 'incorrect user name or password',
        9: 'user not exist',
        101: 'Not Support',
        102: 'destination must be panel or sol',
        103: 'json loads error',

        104: 'load dll error',
        105: 'nic list cannot be null',
        106: 'support up to 2 nics',
        107: 'illegal task id(0-6, 8)'
    }
    ERR_dict_raw = {
        1: 'data acquisition exception',
        2: 'parameter is null',
        3: 'parameter error',
        4: 'create link exception',
        5: 'internal error',
        6: 'allocated memory exception',
        7: 'network connection failed',
        8: 'incorrect user name or password',
        9: 'user not exist'
    }

    def __init__(self):
        pass

    def getfru(self, client, args):
        fru = IpmiFunc.getAllFruByIpmi(client)
        res = ResultBean()
        if fru:
            FRUlist = []
            product = fru
            frubean = collections.OrderedDict()
            if product.get('Product Name', None):
                frubean["FRUName"] = product['Product Name']
            else:
                frubean["FRUName"] = None

            if product['Chassis Type']:
                frubean["ChassisType"] = product['Chassis Type']
            else:
                frubean["ChassisType"] = None

            if product['Chassis Part Number']:
                frubean["ChassisPartNumber"] = product['Chassis Part Number']
            else:
                frubean["ChassisPartNumber"] = None

            if product['Chassis Serial']:
                frubean["ChassisSerial"] = product['Chassis Serial']
            else:
                frubean["ChassisSerial"] = None

            if product['Board Mfg']:
                frubean["BoardMfg"] = product['Board Mfg']
            else:
                frubean["BoardMfg"] = None

            if product['Board Product']:
                frubean["BoardProduct"] = product['Board Product']
            else:
                frubean["BoardProduct"] = None

            if product['Board Serial']:
                frubean["BoardSerial"] = product['Board Serial']
            else:
                frubean["BoardSerial"] = None

            if product['Board Part Number']:
                frubean["BoardPartNumber"] = product['Board Part Number']
            else:
                frubean["BoardPartNumber"] = None

            if product['Product Manufacturer']:
                frubean["ProductManufacturer"] = product['Product Manufacturer']
            else:
                frubean["ProductManufacturer"] = None

            if product['Product Name']:
                frubean["ProductName"] = product['Product Name']
            else:
                frubean["ProductName"] = None

            if product['Product Part Number']:
                frubean["ProductPartNumber"] = product['Product Part Number']
            else:
                frubean["ProductPartNumber"] = None

            if product.get('Product Version', None):
                frubean["ProductVersion"] = product['Product Version']
            else:
                frubean["ProductVersion"] = None

            if product['Product Serial']:
                frubean["ProductSerial"] = product['Product Serial']
            else:
                frubean["ProductSerial"] = None

            if product['Product Asset Tag']:
                frubean["ProductAssetTag"] = product['Product Asset Tag']
            else:
                frubean["ProductAssetTag"] = None

            FRUlist.append(frubean)
            FRU = [{"FRU": FRUlist}]
            res.State('Success')
            res.Message(FRU)
        else:
            res.State('Failure')
            res.Message('Can not get Fru information')
        return res

    def getProdcut(self, client, args):
        """
        :return:
        """

    def getcapabilities(self, client, args):
        """

        :return:
        """
        res = ResultBean()
        cap = CapabilitiesBean()
        getcomand = [
            'get80port',
            'getadaptiveport',
            'getbios',
            'getbiosdebug',
            'getbiosresult',
            'getbiossetting',
            'getcapabilities',
            'getcpu',
            'geteventlog',
            'geteventsub',
            'getfan',
            'getfirewall',
            'getfru',
            'getfw',
            'gethealth',
            'gethealthevent',
            'getip',
            'getldisk',
            'getmemory',
            'getmgmtport',
            'getnic',
            'getpcie',
            'getpdisk',
            'getpower ',
            'getproduct',
            'getpsu',
            'getpwrcap',
            'getraid',
            'getsensor',
            'getserialport',
            'getservice',
            'getsysboot',
            'gettaskstate',
            'gettemp',
            'getthreshold',
            'gettime',
            'gettrap',
            'getupdatestate',
            'getuser',
            'getvnc',
            'getvncsession',
            'getvolt']
        setcommand = [
            'adduser',
            'addwhitelist',
            'canceltask',
            'clearbiospwd',
            'clearsel',
            'collect',
            'deluser',
            'delvncsession',
            'delwhitelist',
            'downloadsol',
            'downloadtfalog',
            'exportbioscfg',
            'exportbmccfg',
            'fancontrol',
            'fwupdate',
            'importbioscfg',
            'importbmccfg',
            'locatedisk',
            'locateserver',
            'mountvmm',
            'powercontrol',
            'powerctrldisk',
            'recoverypsu',
            'resetbmc',
            'restorebios',
            'restorebmc',
            'sendipmirawcmd',
            'settime',
            'settimezone',
            'settrapcom',
            'setadaptiveport',
            'setbios',
            'setbiosdebug',
            'setbiospwd',
            'setfirewall',
            'sethsc',
            'setip',
            'setpriv',
            'setproductserial',
            'setpwd',
            'setserialport',
            'setservice',
            'setsysboot',
            'setthreshold',
            'settrapdest',
            'setvlan',
            'setvnc',
            'setimageurl']
        cap.GetCommandList(getcomand)
        cap.SetCommandList(setcommand)
        res.State('Success')
        res.Message(cap.dict)
        return res

    def getcpu(self, client, args):
        """

        :return:
        """

    def getadaptiveport(self, client, args):
        """

        :param client:
        :param args:
        :return:
        """
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def getmemory(self, client, args):
        """

        :return:
        """

    def powercontrol(self, client, args):
        """
        power control
        :param client:
        :param args:
        :return:power control
        """
        choices = {
            'On': 'on',
            'ForceOff': 'off',
            'ForcePowerCycle': 'cycle',
            'ForceReset': 'reset',
            'GracefulShutdown': 'soft',
            'Nmi': 'diag'}
        # power on, power off(immediately shutdown), power soft(orderly
        # shutdown), power reset, power cycle. Power diag
        ctr_result = ResultBean()
        # 首先获取当前电源状态，如果是关机状态，只能开机操作
        cur_status = IpmiFunc.getPowerStatusByIpmi(client)
        if cur_status and cur_status.get('code') == 0 and cur_status.get(
                'data') is not None and cur_status.get('data').get('status') is not None:
            cur_power = cur_status.get('data').get('status')
            if cur_power == 'off' and (
                    choices[args.state] == 'off' or choices[args.state] == 'soft'):
                ctr_result.State("Success")
                ctr_result.Message('power status is off.')
                return ctr_result
            elif cur_power == 'off' and choices[args.state] != 'on':
                ctr_result.State("Failure")
                ctr_result.Message(
                    'power status is off, please power on first.')
                return ctr_result
        elif cur_status is None:
            ctr_result.State("Failure")
            ctr_result.Message('get power status failed, load dll error')
            return ctr_result
        else:
            ctr_result.State("Failure")
            ctr_result.Message(
                'get power status failed. ' +
                self.ERR_dict.get(
                    cur_status.get('code'),
                    ''))
            return ctr_result
        ctr_info = IpmiFunc.powerControlByIpmi(client, choices[args.state])
        if ctr_info:
            if ctr_info.get('code') == 0 and ctr_info.get(
                    'data') is not None and ctr_info.get('data').get('status') is not None:
                ctr_result.State("Success")
                ctr_result.Message(
                    'set power success,current power status is ' +
                    ctr_info['data'].get('status') +
                    '.')
            else:
                ctr_result.State("Failure")
                ctr_result.Message(
                    'set power failed: ' +
                    ctr_info.get(
                        'data',
                        ' '))
        else:
            ctr_result.State("Failure")
            ctr_result.Message('set power failed.')
        return ctr_result

    def gethealth(self, client, args):
        """

        :return:
        """

    def getsysboot(self, client, args):
        res = ResultBean()
        biosaAttribute = collections.OrderedDict()
        sysboot = IpmiFunc.getSysbootByIpmi(client)
        #{'data': '01 05 00 18 00 00 00', 'code': 0}
        if sysboot['code'] == 0:
            bootflags = sysboot['data'].strip().split(" ")
            data1 = bootflags[2]
            bin_1 = bin(int(data1, 16))[2:]
            if len(bin_1) < 8:
                bin_1 = "0" * (8 - len(bin_1)) + bin_1
            data2 = bootflags[3]
            bin_2 = bin(int(data2, 16))[2:]
            if len(bin_2) < 8:
                bin_2 = "0" * (8 - len(bin_2)) + bin_2
            # boot device
            boot_device = bin_2[2:6]
            boot_device_dict = {'0000': 'none',
                                '0001': 'PXE',
                                '0010': 'HDD',
                                '0011': 'HDD(SafeMode)',
                                '0100': 'Diagnostic Partition',
                                '0101': 'CD',
                                '0110': 'BIOSSETUP',
                                '1111': 'Floppy'}
            biosaAttribute['BootDevice'] = boot_device_dict.get(
                boot_device, 'reserved')
            # boot type
            biosaAttribute['BootMode'] = None
            # apply2
            if bin_1[1] == '0':
                biosaAttribute['Effective'] = 'Once'
            else:
                biosaAttribute['Effective'] = 'Continuous'
            # if bin_1[2]=='0':
            #     biosaAttribute['BootMode']='Legacy'
            # else:
            #     biosaAttribute['BootMode']='UEFI'

            res.State("Success")
            res.Message([biosaAttribute])
        else:
            res.State("Failure")
            res.Message(["get serial port error" + sysboot['data']])
        return res

    def geteventlog(self, client, args):
        """

        :return:
        """

    def downloadtfalog(self, client, args):
        """

        :param client:
        :param args:
        :return:
        """
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def gethealthevent(self, client, args):
        """

        :return:
        """

    def getraid(self, client, args):
        """

        :return:
        """

    def getldisk(self, client, args):
        """

        :return:
        """

    def clearsel(self, client, args):
        """
        clear sel
        :param client:
        :param args:
        :return: clear sel
        """
        clear_result = ResultBean()
        clear_code = IpmiFunc.clearSelByIpmi(client)
        if clear_code == 0:
            clear_result.State("Success")
            clear_result.Message(
                ['Clearing SEL. Please allow a few seconds to erase.'])
        else:
            clear_result.State("Failure")
            clear_result.Message(['Clear sel failed.'])
        return clear_result

    def getnic(self, client, args):
        """

        :return:
        """

    def setsysboot(self, client, args):
        result = ResultBean()
        flag = True
        info = ""
        if args.mode is not None:
            mode_set = IpmiFunc.setBootModeByIpmi(client, args.mode)
            if mode_set['code'] != 0:
                flag = False
                info = "set mode error: " + mode_set['data']
        if args.effective is not None and args.device is not None:
            boot_set = IpmiFunc.setBIOSBootOptionByIpmi(
                client, args.effective, args.device)
            if boot_set['code'] != 0:
                flag = False
                info = info + "set boot device error: " + boot_set['data']
        if flag:
            result.State("Success")
            result.Message([])
        else:
            result.State("Failure")
            result.Message([info])
        return result

    def powerctrldisk(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def powerctrlpcie(self, client, args):
        """

        :return:
        """

    def getbios(self, client, args):
        """

        :return:
        """

    def setbios(self, client, args):
        """

        :return:
        """

    def setbiospwd(self, client, args):
        """

        :return:
        """

    def getbiossetting(self, client, args):
        """

        :return:
        """

    def getbiosresult(self, client, args):
        """

        :return:
        """

    def getbiosdebug(self, client, args):
        """

        :return:
        """

    def clearbiospwd(self, client, args):
        """

        :return:
        """

    def restorebios(self, client, args):
        """

        :return:
        """

    def setbiosdebug(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def mountvmm(self, client, args):
        """

        :return:
        """

    def setthreshold(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    # def downloadfandiag(self, client, args):

    def downloadsol(self, client, args):
        """

        :return:
        """

    def sethsc(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def chassis(self, client, args):
        """

        :return:
        """

    def getproduct(self, client, args):
        """

        :return:
        """

    def setproductserial(self, client, args):
        res = ResultBean()
        serial_set = IpmiFunc.editFruByIpmi(client, 0, 'p', '4', args.serial)
        if serial_set == 0:
            res.State("Success")
            res.Message([])
        else:
            res.State("Failure")
            res.Message(
                [self.ERR_dict.get(serial_set, "set product serial error")])
        return res

    def getfan(self, client, args):
        """

        :return:
        """

    def fancontrol(self, client, args):
        """

        :return:
        """

    def getsensor(self, client, args):
        """
        get sensor info
        :param client:
        :param args:
        :return:
        """
        result = ResultBean()
        sensors_Info = SensorBean()
        sensors = IpmiFunc.getSensorByIpmi(client)

        if sensors:
            if sensors.get('code') == 0 and sensors.get('data') is not None:
                sensorsData = sensors.get('data')
                List = []
                size = len(sensorsData)
                sensors_Info.Maximum(size)
                num = 0
                for sensor in sensorsData:
                    sensor_single = Sensor()
                    # sensor_single.SensorNumber(num)
                    sensor_single.SensorNumber(sensor.get('name', None))
                    sensor_single.SensorName(sensor.get('num', num))
                    if sensor.get('value', None) == '0x0':
                        sensor_single.Reading(float(0))
                    else:
                        sensor_single.Reading(
                            None if sensor.get(
                                'value',
                                None) == 'na' or sensor.get('value') is None else float(
                                sensor.get('value')))
                    sensor_single.Unit(
                        None if sensor.get(
                            'unit', None) == '' else sensor.get(
                            'unit', None))
                    sensor_single.Status(sensor.get('status', None))
                    sensor_single.unr(
                        None if sensor.get(
                            'unr',
                            None) == 'na' or sensor.get(
                            'unr',
                            None) is None or sensor.get(
                            'unr',
                            None) == '' else float(
                            sensor.get('unr')))
                    sensor_single.uc(
                        None if sensor.get(
                            'uc',
                            None) == 'na' or sensor.get(
                            'uc',
                            None) is None or sensor.get(
                            'uc',
                            None) == '' else float(
                            sensor.get('uc')))
                    sensor_single.unc(
                        None if sensor.get(
                            'unc',
                            None) == 'na' or sensor.get(
                            'unc',
                            None) is None or sensor.get(
                            'unc',
                            None) == '' else float(
                            sensor.get('unc')))

                    sensor_single.lnc(
                        None if sensor.get(
                            'lnc',
                            None) == 'na' or sensor.get(
                            'lnc',
                            None) is None or sensor.get(
                            'lnc',
                            None) == '' else float(
                            sensor.get('lnc')))
                    sensor_single.lc(
                        None if sensor.get(
                            'lc',
                            None) == 'na' or sensor.get(
                            'lc',
                            None) is None or sensor.get(
                            'lc',
                            None) == '' else float(
                            sensor.get('lc')))
                    sensor_single.lnr(
                        None if sensor.get(
                            'lnr',
                            None) == 'na' or sensor.get(
                            'lnr',
                            None) is None or sensor.get(
                            'lnr',
                            None) == '' else float(
                            sensor.get('lnr')))
                    num = num + 1
                    List.append(sensor_single.dict)
                sensors_Info.Sensor(List)
                result.State('Success')
                result.Message([sensors_Info.dict])

            else:
                result.State('Failure')
                result.Message(['Failed to get sensor info. ' +
                                self.ERR_dict.get(sensors.get('code'), '')])
        else:
            result.State('Failure')
            result.Message(['Failed to get sensor info, load dll error.'])

        return result

    def getpower(self, client, args):
        """

        :return:
        """

    def gettemp(self, client, args):
        """
        get temperature info
        :param client:
        :param args:
        :return:
        """
        result = ResultBean()
        temp_Info = TemperatureBean()
        sensors_temp = IpmiFunc.getSensorsTempByIpmi(client)
        if sensors_temp:
            if sensors_temp.get('code') == 0 and sensors_temp.get(
                    'data') is not None:
                temps = sensors_temp.get('data')
                # print(temps)
                List = []
                num = 0
                for temp in temps:
                    temp_single = Temperature()
                    if temp.get('unit') == 'degrees C':
                        temp_single.Name(temp.get('name', None))
                        temp_single.SensorNumber(temp.get('num', num))
                        temp_single.UpperThresholdFatal(
                            None if temp.get(
                                'unr',
                                None) == 'na' or temp.get(
                                'unr',
                                None) is None else float(
                                temp.get('unr')))
                        temp_single.UpperThresholdCritical(
                            None if temp.get(
                                'uc',
                                None) == 'na' or temp.get(
                                'uc',
                                None) is None else float(
                                temp.get('uc')))
                        temp_single.UpperThresholdNonCritical(
                            None if temp.get(
                                'unc',
                                None) == 'na' or temp.get(
                                'unc',
                                None) is None else float(
                                temp.get('unc')))
                        if temp.get('value', None) == '0x0':
                            temp_single.ReadingCelsius(float(0))
                        else:
                            temp_single.ReadingCelsius(
                                None if temp.get(
                                    'value',
                                    None) == 'na' or temp.get('value') is None else float(
                                    temp.get('value')))
                        temp_single.LowerThresholdNonCritical(
                            None if temp.get(
                                'lnc',
                                None) == 'na' or temp.get(
                                'lnc',
                                None) is None else float(
                                temp.get('lnc')))
                        temp_single.LowerThresholdCritical(
                            None if temp.get(
                                'lc',
                                None) == 'na' or temp.get(
                                'lc',
                                None) is None else float(
                                temp.get('lc')))
                        temp_single.LowerThresholdFatal(
                            None if temp.get(
                                'lnr',
                                None) == 'na' or temp.get(
                                'lnr',
                                None) is None else float(
                                temp.get('lnr')))
                        num = num + 1
                        List.append(temp_single.dict)
                temp_Info.Temperature(List)
                result.State('Success')
                result.Message([temp_Info.dict])

            else:
                result.State('Failure')
                result.Message(['Failed to get temp info. ' +
                                self.ERR_dict.get(sensors_temp.get('code'), '')])
        else:
            result.State('Failure')
            result.Message(['Failed to get temp info, load dll error.'])

        return result

    def getvolt(self, client, args):
        """
        get voltage info
        :param client:
        :param args:
        :return:
        """
        result = ResultBean()
        temp_Info = VoltBean()
        sensors_temp = IpmiFunc.getSensorsVoltByIpmi(client)
        if sensors_temp:
            if sensors_temp.get('code') == 0 and sensors_temp.get(
                    'data') is not None:
                temps = sensors_temp.get('data')
                # print(temps)
                List = []
                num = 0
                for temp in temps:
                    temp_single = Voltage()
                    if temp.get('unit') == 'Volts':
                        temp_single.Name(temp.get('name', None))
                        temp_single.SensorNumber(temp.get('name', num))
                        temp_single.UpperThresholdFatal(
                            None if temp.get(
                                'unr',
                                None) == 'na' or temp.get(
                                'unr',
                                None) is None else float(
                                temp.get('unr')))
                        temp_single.UpperThresholdCritical(
                            None if temp.get(
                                'uc',
                                None) == 'na' or temp.get(
                                'uc',
                                None) is None else float(
                                temp.get('uc')))
                        temp_single.UpperThresholdNonCritical(
                            None if temp.get(
                                'unc',
                                None) == 'na' or temp.get(
                                'unc',
                                None) is None else float(
                                temp.get('unc')))
                        # temp_single.ReadingVolts(
                        #     None if temp.get('value', None) == 'na' or temp.get('value', None) == '0x0' or temp.get(
                        #         'value') is None else float(temp.get('value')))
                        if temp.get('value', None) == '0x0':
                            temp_single.ReadingVolts(float(0))
                        else:
                            temp_single.ReadingVolts(
                                None if temp.get(
                                    'value',
                                    None) == 'na'or temp.get('value') is None else float(
                                    temp.get('value')))
                        temp_single.LowerThresholdNonCritical(
                            None if temp.get(
                                'lnc',
                                None) == 'na' or temp.get(
                                'lnc',
                                None) is None else float(
                                temp.get('lnc')))
                        temp_single.LowerThresholdCritical(
                            None if temp.get(
                                'lc',
                                None) == 'na' or temp.get(
                                'lc',
                                None) is None else float(
                                temp.get('lc')))
                        temp_single.LowerThresholdFatal(
                            None if temp.get(
                                'lnr',
                                None) == 'na' or temp.get(
                                'lnr',
                                None) is None else float(
                                temp.get('lnr')))

                        num = num + 1
                        List.append(temp_single.dict)
                temp_Info.Voltage(List)
                result.State('Success')
                result.Message([temp_Info.dict])

            else:
                result.State('Failure')
                result.Message(['Failed to get volt info. ' +
                                self.ERR_dict.get(sensors_temp.get('code'), '')])
        else:
            result.State('Failure')
            result.Message(['Failed to get volt info, load dll error.'])

        return result

    def getthreshold(self, client, args):
        """

        :return:
        """

    def getpwrcap(self, client, args):
        """

        :return:
        """

    def getpsu(self, client, args):
        """

        :return:
        """

    def recoverypsu(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def getpdisk(self, client, args):
        """

        :return:
        """

    def getpcie(self, client, args):
        """

        :return:
        """

    def locateserver(self, client, args):
        """

        :return:
        """

    def locatedisk(self, client, args):
        """

        :return:
        """

    def Managers(self, client, args):
        """

        :return:
        """

    def getip(self, client, args):
        """

        :return:
        """

    def getdns(self, client, args):
        """

        :return:
        """

    def gettrap(self, client, args):
        """

        :return:
        """

    def restorebmc(self, client, args):
        """

        :return:
        """

    def collect(self, client, args):
        """

        :return:
        """

    def gettime(self, client, args):
        res = ResultBean()
        offsetres = IpmiFunc.sendRawByIpmi(client, "raw 0x0a 0x5c")
        if offsetres.get("code", 1) == 0:
            offsetraw = offsetres.get("data", "").strip().split(" ")
            offset = int(offsetraw[1] + offsetraw[0], 16)
            we = "+"
            if offset > 32768:
                we = "-"
                offset = 65536 - offset
            hh = str(offset // 60)
            mm = str(offset % 60)
            if len(hh) == 1:
                hh = "0" + hh
            if len(mm) == 1:
                mm = "0" + mm
            timezone = we + str(hh) + ":" + str(mm)
            timeres = IpmiFunc.sendRawByIpmi(client, "raw 0x0a 0x48")
            if timeres.get("code", 1) == 0:
                dataraw = timeres.get("data", "").strip().split(" ")
                timestamp = int(
                    dataraw[3] +
                    dataraw[2] +
                    dataraw[1] +
                    dataraw[0],
                    16)
                timearray = time.gmtime(timestamp)
                showtime = time.strftime("%Y-%m-%d %H:%M:%S", timearray)
                timeinfo = collections.OrderedDict()
                timeinfo['Time'] = showtime
                timeinfo['Timezone'] = timezone
                res.State("Success")
                res.Message(timeinfo)
            else:
                res.State("Success")
                res.Message("cannot get sel time. " + timeres.get("data", ""))
        else:
            res.State("Success")
            res.Message(
                "cannot get sel time UTC offset. " +
                offsetres.get(
                    "data",
                    ""))
        return res

    def settime(self, client, args):
        import time
        res = ResultBean()
        #self.newtime = "2018-05-31T10:10+08:00"
        if not RegularCheckUtil.checkBMCTime(args.newtime):
            res.State("Failure")
            #res.Message(["time param should be like YYYY-mm-ddTHH:MM±HH:MM"])
            res.Message(["time param should be like YYYY-mm-ddTHH:MM+HH:MM"])
            return res
        if "+" in args.newtime:
            newtime = args.newtime.split("+")[0]
            zone = args.newtime.split("+")[1]
            we = "+"
        else:
            zone = args.newtime.split("-")[-1]
            newtime = args.newtime.split("-" + zone)[0]
            we = "-"
        # set zone
        args.zone = "[" + we + zone + "]"
        res_zone = Base.settimezone(self, client, args)
        if res_zone.State == "Failure":
            return res_zone
        # set time
        try:
            #time.struct_time(tm_year=2019, tm_mon=4, tm_mday=16, tm_hour=15, tm_min=35, tm_sec=0, tm_wday=1, tm_yday=106, tm_isdst=-1)
            structtime = time.strptime(newtime, "%Y-%m-%dT%H:%M")
            # 时间戳1555400100
            stamptime = int(time.mktime(structtime))
        except ValueError as e:
            res.State("Failure")
            res.Message(["illage time param"])
            return res
        # 执行
        time_set = IpmiFunc.setBMCTimeByIpmi(client, stamptime)

        if time_set["code"] == 0:
            res.State("Success")
            res.Message([])
        else:
            res.State("Failure")
            res.Message(["set time error" + time_set.get('data', '')])
        return res

    def settimezone(self, client, args):
        res = ResultBean()
        if RegularCheckUtil.checkBMCZone(args.zone) == 1:
            res.State("Failure")
            #res.Message(["timezone should be like [±HH:MM]"])
            res.Message(["timezone should be like [+HH:MM]"])
            return res
        elif RegularCheckUtil.checkBMCZone(args.zone) == 2:
            res.State("Failure")
            res.Message(["timezone should be -12:00 to +14:00"])
            return res
        ew = args.zone[1]
        hh = int(args.zone[2:4])
        mm = int(args.zone[5:7])
        zoneMinutes = int(ew + str(hh * 60 + mm))
        if mm != 0 and mm != 30:
            res.State("Not Support")
            res.Message(["minutes can only be 0 or 30 now"])
            return res

        time_set = IpmiFunc.setBMCTimezoneByIpmi(client, zoneMinutes)
        if time_set["code"] == 0:
            res.State("Success")
            res.Message([])
        else:
            res.State("Failure")
            res.Message(["set time zone error" + time_set.get('data', '')])
        return res

    def resetbmc(self, client, args):
        """

        :return:
        """
        res = ResultBean()
        reset = IpmiFunc.resetBMCByIpmi(client)
        if reset == 0:
            res.State("Success")
            res.Message([])
        else:
            res.State("Failure")
            res.Message(["reset bmc failure"])
        return res

    def setip(self, client, args):
        """

        :return:
        """

    def setvlan(self, client, args):
        """

        :return:
        """

    def getvnc(self, client, args):
        """

        :return:
        """

    def setvnc(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def getservice(self, client, args):
        """

        :return:
        """

    def setservice(self, client, args):
        """

        :return:
        """

    def getmgmtport(self, client, args):
        """

        :param client:
        :param args:
        :return:
        """
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def getserialport(self, client, args):
        """

        :return:
        """

    def setserialport(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def setadaptiveport(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def settrapcom(self, client, args):
        """

        :return:
        """

    def settrapdest(self, client, args):
        """

        :return:
        """

    def exportbmccfg(self, client, args):
        """

        :return:
        """

    def importbmccfg(self, client, args):
        """

        :return:
        """

    def exportbioscfg(self, client, args):
        """
        export bios setup configuration
        :param client:
        :param args:
        :return:
        """
        bios = ResultBean()
        bios.State('Not Support')
        bios.Message([])
        return bios

    def importbioscfg(self, client, args):
        """
        import bios cfg
        :param client:
        :param args:
        :return:
        """
        bios = ResultBean()
        bios.State('Not Support')
        bios.Message([])
        return bios

    def delvncsession(self, client, args):
        """

        :return:
        """

    def sendipmirawcmd(self, client, args):
        """

        :return:
        """

    def AccountService(self, client, args):
        """

        :return:
        """

    def getuser(self, client, args):
        """

        :return:
        """

    def adduser(self, client, args):
        """

        :return:
        """

    def deluser(self, client, args):
        """

        :return:
        """

    def setpwd(self, client, args):
        res = ResultBean()
        result = IpmiFunc.getUserByIpmi(client)
        if result['code'] != 0:
            res.State("Failure")
            res.Message([result['data']])
            return res
        userlist = result['data']
        flag = False
        for item in userlist:
            if not item:
                continue
            if item['UserName'] == args.uname:
                flag = True
                user = item
                break
        if not flag:
            res.State("Failure")
            res.Message(["user not exits"])
            return res
        userRes = IpmiFunc.setUserPassByIpmi(client, user['UserId'], args.upass)
        if userRes == 0:
            res.State("Success")
            res.Message([])
        else:
            res.State("Failure")
            res.Message(["user not exits"])
        return res

    def setpriv(self, client, args):
        """

        :return:
        """

    def updateservice(self, client, args):
        """

        :return:
        """

    def getfw(self, client, args):
        """

        :return:
        """

    def getupdatestate(self, client, args):
        """

        :return:
        """

    def fwupdate(self, client, args):
        """

        :return:
        """

    def getvncsession(self, client, args):
        """

        :return:
        """

    def EventServices(self, client, args):
        """

        :return:"""

    def geteventsub(self, client, args):
        """

        :param client:
        :param args:
        :return:
        """
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def addeventsub(self, client, args):
        """

        :return:
        """

    def deleventsub(self, client, args):
        """

        :return:
        """

    def TaskService(self, client, args):
        """

        :return:
        """

    def gettaskstate(self, client, args):
        """

        :return:
        """

    def cancletask(self, client, args):
        """

        :return:
        """

    def get80port(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def getfirewall(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def setfirewall(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def Opwhitelist(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def addwhitelist(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def delwhitelist(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def setimageurl(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def getimageurl(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def methods(self, client, args):
        return(list(filter(lambda m: not m.startswith("__") and not m.endswith("__") and callable(getattr(self, client, args, m)), dir(self, client, args))))

    def setbmclogcfg(self, client, args):
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def remoteFWUpdate(self, client, args):
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def updatepsu(self, client, args):
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def gethba(self, client, args):
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def getSystemLockdownMode(self, client, args):
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

    def setSystemLockdownMode(self, client, args):
        result = ResultBean()
        result.State("Not Support")
        result.Message([])
        return result

# Ascii转十六进制


def ascii2hex(data, length):
    count = length - len(data)
    list_h = []
    for c in data:
        list_h.append(str(hex(c)))
    data = ' '.join(list_h) + ' 0x00' * count
    return data

# 十六进制字符串逆序


def hexReverse(data):
    pattern = re.compile('.{2}')
    time_hex = ' '.join(pattern.findall(data))
    seq = time_hex.split(' ')[::-1]
    data = '0x' + ' 0x'.join(seq)
    return data



# -*- coding:utf-8 -*-

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import sys
import json
import signal
from importlib import import_module
from inmanage_sdk.command import RestFunc
import time
import collections
try:
    from inmanage_sdk.util import configUtil, HostTypeJudge, parameterConversion, RequestClient

    INMANAGE_EXIST = True
except ImportError:
    INMANAGE_EXIST = False
sys.path.append(os.path.join(sys.path[0], "interface"))
current_time = time.strftime(
    '%Y-%m-%d   %H:%M:%S',
    time.localtime(
        time.time()))
__version__ = '1.2.1'


ERR_dict = {
    'ERR_CODE_CMN_FAIL': 'data acquisition exception',
    'ERR_CODE_PARAM_NULL': 'parameter is null',
    'ERR_CODE_INPUT_ERROR': 'parameter error',
    'ERR_CODE_INTF_FAIL': 'create link exception',
    'ERR_CODE_INTERNAL_ERROR': 'internal error',
    'ERR_CODE_ALLOC_MEM': 'allocated memory exception',
    'ERR_CODE_NETWORK_CONNECT_FAIL': 'network connection failed',
    'ERR_CODE_AUTH_NAME_OR_PWD_ERROR': 'incorrect user name or password',
    'ERR_CODE_USER_NOT_EXIST': 'user not exist'
}


def main(params):

    def logout(signum, frame):
        if hasattr(client, "header"):
            RestFunc.logout(client)

    signal.signal(signal.SIGINT, logout)
    signal.signal(signal.SIGTERM, logout)
    signal.signal(signal.SIGABRT, logout)
    # windows下注释下面两行
    signal.signal(signal.SIGHUP, logout)
    signal.signal(signal.SIGQUIT, logout)
    res = {}
    if not INMANAGE_EXIST:
        res['State'] = "Failure"
        res['Message'] = ["Please install the requests library"]
        return res
    param = parameterConversion.getParam(params)
    args = dict_to_object(param)
    args.port = None
    configutil = configUtil.configUtil()
    if args.subcommand is not None and args.subcommand == 'support_model':
        res['State'] = "Success"
        res['Message'] = configutil.getModelSupport()
        return res
    # 使用fru获取机型信息
    hostTypeClient = HostTypeJudge.HostTypeClient()
    productName, firmwareVersion = hostTypeClient.getProductNameByIPMI(args)
    if productName is None:
        res['State'] = "Not Support"
        res['Message'] = ["cannot get productName"]
        return res
    elif productName in ERR_dict:
        res['State'] = "Failure"
        res['Message'] = [ERR_dict.get(productName)]
        return res
    if firmwareVersion is None:
        res['State'] = "Failure"
        res['Message'] = ["cannot get BMC version"]
        return res
    configutil = configUtil.configUtil()
    impl, platform = configutil.getRouteOption(productName, firmwareVersion)
    args.productName = productName
    args.platform = platform
    if 'Error' in impl:
        res['State'] = "Failure"
        res['Message'] = [impl]
        return res
    module_impl = 'inmanage_sdk.interface.' + impl
    obj = import_module(module_impl)
    targetclass = getattr(obj, impl)
    obj = targetclass()
    if args.subcommand is None:
        res['State'] = "Failure"
        res['Message'] = ["please input a subcommand"]
        return res
    targetMed = getattr(obj, args.subcommand)
    client = RequestClient.RequestClient()
    client.setself(
        args.host,
        args.username,
        args.passcode,
        platform,
        args.port,
        'lanplus')
    try:
        resultJson = targetMed(client, args)
    except Exception as e:
        # 保留日志
        # import traceback
        # utool_path = os.path.dirname(os.path.abspath(__file__))
        # # print(utool_path)
        # log_path = os.path.join(utool_path, "log")
        # if not os.path.exists(log_path):
        #     os.makedirs(log_path)
        # # TIME
        # localtime = time.localtime()
        # f_localdate = time.strftime("%Y-%m-%d", localtime)
        # f_localtime = time.strftime("%Y-%m-%dT%H:%M:%S ", localtime)
        #
        # log_file = os.path.join(log_path, f_localdate)
        # args.items()
        # res_info = "[" + args.subcommand + "]" + traceback.format_exc()
        # with open(log_file, 'a+') as logfile:
        #     utoollog = "[ERROR]" + f_localtime + res_info + json.dumps(param, default=lambda o: o.__dict__, indent=4, ensure_ascii=True)
        #     logfile.write(utoollog)
        #     logfile.write("\n")

        res['State'] = "Failure"
        res['Message'] = ["Error occurs, request failed..."]
        return res
    sortedRes = collections.OrderedDict()
    sortedRes["State"] = resultJson.State
    sortedRes["Message"] = resultJson.Message
    return sortedRes


class Dict(dict):
    __setattr__ = dict.__setitem__
    __getattr__ = dict.__getitem__


def dict_to_object(dictobj):
    if not isinstance(dictobj, dict):
        return dictobj
    inst = Dict()
    for k, v in dictobj.items():
        if k == 'password':
            k = 'passcode'
        inst[k] = dict_to_object(v)
    return inst



# -*- coding:utf-8 -*-

eventLogString = {}

eventLogString["en"] = {
    'STR_EVENT_LOG_SOFTWARE_ID': "System Software ID",
    'STR_EVENT_LOG_TYPE': {
        0: "All Events",
        1: "System Event Records",
        2: "OEM Event Records",
        3: "BIOS Generated Events",
        4: "SMI Handler Events",
        5: "System Management Software Events",
        6: "System Software - OEM Events",
        7: "Remote Console software Events",
        8: "Terminal Mode Remote Console software Events"
    },
    'STR_EVENT_LOG_ASSERT': "Asserted",
    'STR_EVENT_LOG_DEASSERT': "Deasserted",
    'STR_EVENT_LOG_INVALID_OFFSET': "Invalid Offset for this SensorType"
}


# -*- coding:utf-8 -*-

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from inmanage_sdk.interface.CommonM5 import CommonM5
from inmanage_sdk.command import RestFunc
from inmanage_sdk.interface.ResEntity import (ResultBean, CapabilitiesBean, NicAllBean, NICBean, NICController, NicPort)
import os
import random
import time

STR_PD_DEV_SPEED = {
    0: 0,
    1: 1.5,
    2: 3.0,
    3: 6.0,
    4: 12.0
}

# STR_LD_ACCESS = {
#     0:'Read Write',
#     2:'Read Only',
#     3:'Blocked',
#     15:'Hidden'
# }
stripSizes = {
    7: '64k',
    8: '128k',
    9: '256k',
    10: '512k',
    11: '1024k'

}
accessPolicys = {
    1: 'Read Write',
    2: 'Read Only',
    3: 'Blocked'
}
readPolicys = {
    1: 'Read Ahead',
    2: 'No Read Ahead'
}
writePolicys = {
    1: 'Write Throgh',
    2: 'Write Back',
    3: 'Write caching ok if bad BBU'
}
ioPolicys = {
    1: 'Direct IO',
    2: 'Cached IO'
}

driveCaches = {
    1: 'Unchanged',
    2: 'Enabled',
    3: 'Disabled'

}
initStates = {
    1: 'No Init',
    2: 'Quick Init',
    3: 'Full Init'
}
State = {
    0: 'Offline',
    1: 'Partially Degraded',
    2: 'Degraded',
    3: 'Optimal'
}


class NF5180M5(CommonM5):

    def getcapabilities(self, client, args):
        res = ResultBean()
        cap = CapabilitiesBean()
        getcomand = [
            'getadaptiveport',
            'getbios',
            'getcapabilities',
            'getcpu',
            'geteventlog',
            'getfan',
            'getfru',
            'getfw',
            'gethealth',
            'getip',
            'getnic',
            'getpcie',
            'getpdisk',
            'getpower',
            'getproduct',
            'getpsu',
            'getsensor',
            'getservice',
            'getsysboot',
            'gettemp',
            'gettime',
            'gettrap',
            'getuser',
            'getvolt',
            'getraid',
            'getmemory',
            'getldisk',
            'getfirewall',
            'gethealthevent']
        getcomand_not_support = [
            'getbiossetting',
            'getbiosresult',
            'geteventsub',
            'getpwrcap',
            'getmgmtport',
            'getupdatestate',
            'getserialport',
            'getvnc',
            'getvncsession',
            'gettaskstate',
            'getbiosdebug',
            'getthreshold',
            'get80port']
        setcommand = [
            'adduser',
            'clearsel',
            'deluser',
            'fancontrol',
            'fwupdate',
            'locatedisk',
            'locateserver',
            'mountvmm',
            'powercontrol',
            'resetbmc',
            'restorebmc',
            'sendipmirawcmd',
            'settimezone',
            'settrapcom',
            'setbios',
            'setip',
            'setpriv',
            'setpwd',
            'setservice',
            'setsysboot',
            'settrapdest',
            'setvlan',
            'settime',
            'setproductserial']
        setcommand_ns = [
            'setbiospwd',
            'sethsc',
            'clearbiospwd',
            'restorebios',
            'setfirewall',
            'setimageurl',
            'setadaptiveport',
            'setserialport',
            'powerctrldisk',
            'recoverypsu',
            'setvnc',
            'downloadtfalog',
            'setthreshold',
            'addwhitelist',
            'delwhitelist',
            'delvncsession',
            'downloadsol',
            'exportbmccfg',
            'exportbioscfg',
            'importbioscfg',
            'importbmccfg',
            'canceltask',
            'setbiosdebug',
            'collect']
        cap.GetCommandList(getcomand)
        cap.SetCommandList(setcommand)
        res.State('Success')
        res.Message(cap)
        return res

    def getnic(self, client, args):
        # login
        headers = RestFunc.login(client)
        client.setHearder(headers)
        try:
            # get
            res = RestFunc.getAdapterByRest(client)
            nicRes = ResultBean()
            if res == {}:
                nicRes.State("Failure")
                nicRes.Message(["cannot get information"])
            elif res.get('code') == 0 and res.get('data') is not None:
                port_status_dict = {0: "Not Linked", 1: "Linked", 2: "NA"}
                nicRes.State("Success")
                nicinfo = NicAllBean()
                nicinfo.OverallHealth("OK")
                nicinfo.Maximum(1)
                PCIEinfo = NICBean()
                PCIEinfo.CommonName("PCIE")
                PCIEinfo.Location(None)
                PCIEinfo.Manufacturer(None)
                PCIEinfo.Model(None)
                PCIEinfo.Serialnumber(None)
                PCIEinfo.State(None)
                PCIEinfo.Health(None)
                sys_adapters = res.get('data')['sys_adapters']
                controllerList = []
                for ada in sys_adapters:
                    for adaport in ada['ports']:
                        adapterinfo = NICController()
                        adapterinfo.Id(adaport['id'])
                        adapterinfo.Manufacturer(ada['vendor'])
                        adapterinfo.Model(ada['model'])
                        adapterinfo.Serialnumber(None)
                        adapterinfo.FirmwareVersion(None)
                        adapterinfo.PortCount(adaport['port_num'])
                        portlist = []
                        for x in range(4):
                            portBean = NicPort()
                            portBean.Id(x + 1)
                            if x == 0:
                                x = ""
                            portBean.MACAddress(adaport['mac_addr' + str(x)])
                            portBean.LinkStatus(None)
                            portBean.MediaType(None)
                            portlist.append(portBean.dict)
                        adapterinfo.Port(portlist)
                        controllerList.append(adapterinfo.dict)
                PCIEinfo.Controller(controllerList)
                nicinfo.NIC([PCIEinfo.dict])
                nicRes.Message([nicinfo.dict])
            elif res.get('code') == 0 and res.get('data') is not None:
                nicRes.State("Failure")
                nicRes.Message([res.get('data')])
            else:
                nicRes.State("Failure")
                nicRes.Message(
                    ["get information error, error code " + str(res.get('code'))])
            # logout
            RestFunc.logout(client)
            return nicRes
        except BaseException:
            RestFunc.logout(client)
            return CommonM5.getnic(self, client, args)

    def collect(self, client, args):
        res = ResultBean()
        res.State("Not Support")
        res.Message([])
        return res

    def collectblackbox(self, client, args):
        # login
        headers = RestFunc.login(client)
        if headers == {}:
            login_res = ResultBean()
            login_res.State("Failure")
            login_res.Message(
                ["login error, please check username/password/host/port"])
            return login_res
        client.setHearder(headers)
        bmcres = ResultBean()
        try:
            file_path = os.path.dirname(args.fileurl)
            randomID = str(random.randint(0, 1000))
            bbl_path = os.path.join(file_path, randomID)
            while os.path.exists(bbl_path):
                randomID = str(random.randint(0, 1000))
                bbl_path = os.path.join(file_path, randomID)
            if not os.path.exists(bbl_path):
                os.makedirs(bbl_path)
                time.sleep(5)

            self.collectBlackboxlog(client, bbl_path, "blackbox")
            self.collectBlackboxlog(client, bbl_path, "blackboxpeci")
            # 打包
            import tarfile
            tar = tarfile.open(args.fileurl, "w")
            for root, dir, files in os.walk(bbl_path):
                for file in files:
                    fullpath = os.path.join(root, file)
                    tar.add(fullpath, arcname=file)
            tar.close()

            for root, dir, files in os.walk(bbl_path):
                for file in files:
                    fullpath = os.path.join(root, file)
                    os.remove(fullpath)

            if os.path.exists(bbl_path):
                os.removedirs(bbl_path)

            bmcres.State("Success")
            bmcres.Message(
                ["Download blackbox log success, the file path is " + os.path.abspath(args.fileurl)])
            # logout
            RestFunc.logout(client)
            return bmcres
        except Exception as e:
            bmcres.State("Failure")
            bmcres.Message(["Cannot collect blackbox log: " + str(e)])
            return bmcres

    # 收集日志到bbl_path
    def collectBlackboxlog(self, client, bbl_path, logtype):  # 查看黑盒日志是否存在
        bbl_file = os.path.join(bbl_path, logtype + ".log")
        log_exist_res = RestFunc.getblacklogfileexist(client, logtype)
        if log_exist_res.get("code") == 0:
            if log_exist_res.get("data").get('FileExistFlag') == 1:
                download_res = RestFunc.downloadBlackboxlogByRest(
                    client, bbl_file, logtype)
                if download_res.get("code") == 0:
                    a = ""
                    with open(download_res.get("data"), 'rb') as f:
                        a = f.read()
                    try:
                        b = a.decode("utf-8")
                    except BaseException:
                        # 需要解码
                        import platform
                        if platform.system() == 'Linux' or platform.system() == 'Darwin':
                            cmd = '/tools//blackbox_decrypt/blackbox_decrypt '
                        else:
                            cmd = "\\tools\\blackbox_decrypt\\blackbox_decrypt.exe "
                        cmd = os.path.dirname(os.path.dirname(
                            __file__)) + cmd + download_res.get("data") + " > " + os.path.join(bbl_path, "res")
                        try:
                            pathnow = os.getcwd()
                            os.chdir(os.path.dirname(download_res.get("data")))
                            result_cmd = os.popen(cmd)
                            result_cmd.close()
                            with open(os.path.join(bbl_path, "res")) as deres:
                                deresTxt = deres.read()
                            if deresTxt.find('Success') > -1:
                                logNameStart = deresTxt.find(
                                    'Decrypt Success to')
                                logName = deresTxt[logNameStart + 18:].strip()
                                if logName[-1] == '.':
                                    logName = logName[:-1]
                                import shutil
                                shutil.move(logName, download_res.get("data"))
                            os.chdir(pathnow)
                            if os.path.exists(os.path.join(bbl_path, "res")):
                                os.remove(os.path.join(bbl_path, "res"))
                        except Exception as e:
                            return

    def exportbioscfg(self, client, args):
        '''
        export bios setup configuration
        :param client:
        :param args:
        :return:
        '''
        export = ResultBean()

        file_path = os.path.dirname(args.fileurl)
        file_name = os.path.basename(args.fileurl)

        if not os.path.exists(file_path):
            try:
                os.makedirs(file_path)
            except BaseException:
                export.State("Failure")
                export.Message(["cannot build path."])
                return export
        if '.json' not in file_name and '.conf' not in file_name:
            export.State("Failure")
            export.Message(["please input filename with suffix .json/.conf."])
            return export
        # login
        headers = RestFunc.login(client)
        if headers == {}:
            login_res = ResultBean()
            login_res.State("Failure")
            login_res.Message(
                ["login error, please check username/password/host/port"])
            return login_res
        client.setHearder(headers)
        # get
        res = RestFunc.exportTwoBiosCfgByRest(client, args.fileurl, file_name)

        if res == {}:
            export.State("Failure")
            export.Message(["export bios setup configuration file failed."])
        elif res.get('code') == 0:
            export.State('Success')
            export.Message([res.get('data')])
        elif res.get('code') == 4:
            export.State('Failure')
            export.Message([res.get('data')])
        else:
            export.State("Failure")
            export.Message(
                ["export bios setup configuration file error, " + res.get('data')])
        # logout
        RestFunc.logout(client)
        return export

    def importbioscfg(self, client, args):
        """
        import bios cfg
        :param client:
        :param args:
        :return:
        """
        # login
        headers = RestFunc.login(client)
        if headers == {}:
            login_res = ResultBean()
            login_res.State("Failure")
            login_res.Message(
                ["login error, please check username/password/host/port"])
            return login_res
        client.setHearder(headers)
        # get
        res = RestFunc.importTwoBiosCfgByRest(client, args.fileurl)
        import_Info = ResultBean()
        if res == {}:
            import_Info.State("Failure")
            import_Info.Message(
                ["import bios setup configuration file failed."])
        elif res.get('code') == 0:
            import_Info.State('Success')
            import_Info.Message(
                ['import bios setup configuration file success.'])
        else:
            import_Info.State("Failure")
            import_Info.Message(
                ["import bios setup configuration failed, " + str(res.get('data'))])
        # logout
        RestFunc.logout(client)
        return import_Info



# -*- coding:utf-8 -*-

sensorSpecificEventStr_EN = {}

sensorSpecificEventStr_EN[0] = "reserved"
sensorSpecificEventStr_EN[1] = "Temperature"
sensorSpecificEventStr_EN[2] = "Voltage"
sensorSpecificEventStr_EN[3] = "Current"
sensorSpecificEventStr_EN[4] = "Fan"

sensorSpecificEventStr_EN[5] = {}
sensorSpecificEventStr_EN[5][0] = "General Chassis Intrusion"
sensorSpecificEventStr_EN[5][1] = "Drive Bay Intrusion"
sensorSpecificEventStr_EN[5][2] = "I/O Card Area Intrusion"
sensorSpecificEventStr_EN[5][3] = "Processor Area Intrusion"
sensorSpecificEventStr_EN[5][4] = "LAN Leash Lost (System unplugged from LAN)"
sensorSpecificEventStr_EN[5][5] = "Unauthorized Dock/Undock"
sensorSpecificEventStr_EN[5][6] = "Fan Area Intrusion"

sensorSpecificEventStr_EN[6] = {}
sensorSpecificEventStr_EN[6][0] = "Secure Mode Violation Attempt"
sensorSpecificEventStr_EN[6][1] = "Pre-boot Password Violation - user password"
sensorSpecificEventStr_EN[6][2] = "Pre-boot Password Violation Attempt - setup password"
sensorSpecificEventStr_EN[6][3] = "Pre-boot Password Violation - network boot password"
sensorSpecificEventStr_EN[6][4] = "Other Pre-boot Password Violation"
sensorSpecificEventStr_EN[6][5] = "Out-of-Band Access Password Violation"

sensorSpecificEventStr_EN[7] = {}
sensorSpecificEventStr_EN[7][0] = "IERR"
sensorSpecificEventStr_EN[7][1] = "Thermal Trip"
sensorSpecificEventStr_EN[7][2] = "FRB1/BIST Failure"
sensorSpecificEventStr_EN[7][3] = "FRB2/Hang in POST Faliure"
sensorSpecificEventStr_EN[7][4] = "FRB3/Processor Startup/Initialization Failure"
sensorSpecificEventStr_EN[7][5] = "Configuration Error (DMI)"
sensorSpecificEventStr_EN[7][6] = "SM BIOS 'Uncorrectable CPU-complex Error"
sensorSpecificEventStr_EN[7][7] = "Processor Presence Detected"
sensorSpecificEventStr_EN[7][8] = "Processor Disabled"
sensorSpecificEventStr_EN[7][9] = "Terminator Presence Detected"
sensorSpecificEventStr_EN[7][10] = "Processor Automatically Throttled"
sensorSpecificEventStr_EN[7][11] = "Machine Check Exception"
sensorSpecificEventStr_EN[7][12] = "Correctable Machine Check Error"

sensorSpecificEventStr_EN[8] = {}
sensorSpecificEventStr_EN[8][0] = "Presence Detected"
sensorSpecificEventStr_EN[8][1] = "Power Supply Failure Detected"
sensorSpecificEventStr_EN[8][2] = "Predictive Failure Asserted"
sensorSpecificEventStr_EN[8][3] = "Power Supply Input Lost (AC/DC)"
sensorSpecificEventStr_EN[8][4] = "Power Supply Input Lost or Out of Range"
sensorSpecificEventStr_EN[8][5] = "Power Supply Input Out of Range, but Present"
sensorSpecificEventStr_EN[8][6] = "Configuration Error"
sensorSpecificEventStr_EN[8][7] = "PSU Power Supply Lost"
sensorSpecificEventStr_EN[8][8] = "PCH Power Supply Lost"

sensorSpecificEventStr_EN[8][9] = "PSU B0 Module Power Supply Lost"
sensorSpecificEventStr_EN[8][10] = "PSU B2 Module Power Supply Lost"
sensorSpecificEventStr_EN[8][11] = "PSU B4 Module Power Supply Lost"
sensorSpecificEventStr_EN[8][12] = "PSU B6 Module Power Supply Lost"

sensorSpecificEventStr_EN[9] = {}
sensorSpecificEventStr_EN[9][0] = "Power Off / Power Down"
sensorSpecificEventStr_EN[9][1] = "Power Cycle"
sensorSpecificEventStr_EN[9][2] = "240VA Power Down"
sensorSpecificEventStr_EN[9][3] = "Interlock Power Down"
sensorSpecificEventStr_EN[9][4] = "AC Lost / Power input lost"
sensorSpecificEventStr_EN[9][5] = "Soft Power Control Failure"
sensorSpecificEventStr_EN[9][6] = "Power Unit Failure Detected"
sensorSpecificEventStr_EN[9][7] = "Predictive Failure"

sensorSpecificEventStr_EN[12] = {}
sensorSpecificEventStr_EN[12][0] = "Correctable ECC"
sensorSpecificEventStr_EN[12][1] = "Uncorrectable ECC"
sensorSpecificEventStr_EN[12][2] = "Parity"
sensorSpecificEventStr_EN[12][3] = "Memory Scrub Failure"
sensorSpecificEventStr_EN[12][4] = "Memory Device Disabled"
sensorSpecificEventStr_EN[12][5] = "Correctable ECC Logging Limit Reached"
sensorSpecificEventStr_EN[12][6] = "Presence Detected"
sensorSpecificEventStr_EN[12][7] = "Configuration Error"
sensorSpecificEventStr_EN[12][8] = "Spare"
sensorSpecificEventStr_EN[12][9] = "Memory Automatically Throttled"
sensorSpecificEventStr_EN[12][10] = "Critical Overtemperature"

sensorSpecificEventStr_EN[13] = {}
sensorSpecificEventStr_EN[13][0] = "Drive Presence"
sensorSpecificEventStr_EN[13][1] = "Drive Fault"
sensorSpecificEventStr_EN[13][2] = "Predictive Failure"
sensorSpecificEventStr_EN[13][3] = "Hot Spare"
sensorSpecificEventStr_EN[13][4] = "Consistency Check / Parity Check in progress"
sensorSpecificEventStr_EN[13][5] = "In Critical Array"
sensorSpecificEventStr_EN[13][6] = "In Failed Array"
sensorSpecificEventStr_EN[13][7] = "Rebuild/Remap in progress"
sensorSpecificEventStr_EN[13][8] = "Rebuild/Remap Aborted"

sensorSpecificEventStr_EN[15] = {}
sensorSpecificEventStr_EN[15][0] = "Error"
sensorSpecificEventStr_EN[15][1] = "Hang"
sensorSpecificEventStr_EN[15][2] = "Progress"

sensorSpecificEventStr_EN[16] = {}
sensorSpecificEventStr_EN[16][0] = "Correctable Memory Error Logging Disabled"
sensorSpecificEventStr_EN[16][1] = "Event 'Type' Logging Disabled"
sensorSpecificEventStr_EN[16][2] = "Log Area Reset/Cleared"
sensorSpecificEventStr_EN[16][3] = "All Event Logging Disabled"
sensorSpecificEventStr_EN[16][4] = "SEL Full"
sensorSpecificEventStr_EN[16][5] = "SEL Almost Full"
sensorSpecificEventStr_EN[16][6] = "Correctable Machine Check Error Logging Disabled"

sensorSpecificEventStr_EN[17] = {}
sensorSpecificEventStr_EN[17][0] = "BIOS Watchdog Reset"
sensorSpecificEventStr_EN[17][1] = "OS Watchdog Reset"
sensorSpecificEventStr_EN[17][2] = "OS Watchdog Shutdown"
sensorSpecificEventStr_EN[17][3] = "OS Watchdog Power Down"
sensorSpecificEventStr_EN[17][4] = "OS Watchdog Power Cycle"
sensorSpecificEventStr_EN[17][5] = "OS Watchdog NMI"
sensorSpecificEventStr_EN[17][6] = "OS Watchdog Expired"
sensorSpecificEventStr_EN[17][7] = "OS Watchdog Pre-timeout Interrupt, non-NMI"

sensorSpecificEventStr_EN[18] = {}
sensorSpecificEventStr_EN[18][0] = "System Reconfigured"
sensorSpecificEventStr_EN[18][1] = "OEM System Boot Event"
sensorSpecificEventStr_EN[18][2] = "Undetermined System Hardware Failure"
sensorSpecificEventStr_EN[18][3] = "Entry Added to Auxiliary Log"
sensorSpecificEventStr_EN[18][4] = "PEF Action"
sensorSpecificEventStr_EN[18][5] = "SEL Time Update(OS)"

sensorSpecificEventStr_EN[19] = {}
sensorSpecificEventStr_EN[19][0] = "Front Panel NMI"
sensorSpecificEventStr_EN[19][1] = "Bus Timeout"
sensorSpecificEventStr_EN[19][2] = "I/O Channel Check NMI"
sensorSpecificEventStr_EN[19][3] = "Software NMI"
sensorSpecificEventStr_EN[19][4] = "PCI PERR"
sensorSpecificEventStr_EN[19][5] = "PCI SERR"
sensorSpecificEventStr_EN[19][6] = "EISA Fail Safe Timeout"
sensorSpecificEventStr_EN[19][7] = "Bus Correctable Error"
sensorSpecificEventStr_EN[19][8] = "Bus Uncorrectable Error"
sensorSpecificEventStr_EN[19][9] = "Fatal NMI"
sensorSpecificEventStr_EN[19][10] = "Bus Fatal Error"
sensorSpecificEventStr_EN[19][11] = "Bus Degraded"

sensorSpecificEventStr_EN[20] = {}
sensorSpecificEventStr_EN[20][0] = "Power Button Pressed"
sensorSpecificEventStr_EN[20][1] = "Sleep Button Pressed"
sensorSpecificEventStr_EN[20][2] = "Reset Button Pressed"
sensorSpecificEventStr_EN[20][3] = "FRU Latch Open"
sensorSpecificEventStr_EN[20][4] = "FRU Service Request Button"

sensorSpecificEventStr_EN[25] = {}
sensorSpecificEventStr_EN[25][0] = "Soft Power Control Failure"
sensorSpecificEventStr_EN[25][1] = "Thermal Trip"

sensorSpecificEventStr_EN[27] = {}
sensorSpecificEventStr_EN[27][0] = "Cable/Interconnect is connected"
sensorSpecificEventStr_EN[27][1] = "Configuration Error"
sensorSpecificEventStr_EN[27][2] = "Cable/Interconnect is not connected"

sensorSpecificEventStr_EN[29] = {}
sensorSpecificEventStr_EN[29][0] = "Initiated By Power Up"
sensorSpecificEventStr_EN[29][1] = "Initiated By Hard Reset"
sensorSpecificEventStr_EN[29][2] = "Initiated By Warm Reset"
sensorSpecificEventStr_EN[29][3] = "User Requested PXE Boot"
sensorSpecificEventStr_EN[29][4] = "Automatic Boot to Diagnostic"
sensorSpecificEventStr_EN[29][5] = "OS / run-time software initiated hard reset"
sensorSpecificEventStr_EN[29][6] = "OS / run-time software initiated warm reset"
sensorSpecificEventStr_EN[29][7] = "System Restart"

sensorSpecificEventStr_EN[30] = {}
sensorSpecificEventStr_EN[30][0] = "No Bootable Media"
sensorSpecificEventStr_EN[30][1] = "Non-Bootable Diskette Left in Drive"
sensorSpecificEventStr_EN[30][2] = "PXE Server Not Found"
sensorSpecificEventStr_EN[30][3] = "Invalid Boot Sector"
sensorSpecificEventStr_EN[30][4] = "Timeout Waiting For User Selection of Boot Source"

sensorSpecificEventStr_EN[31] = {}
sensorSpecificEventStr_EN[31][0] = "A: Boot Completed"
sensorSpecificEventStr_EN[31][1] = "C: Boot Completed"
sensorSpecificEventStr_EN[31][2] = "PXE Boot Completed"
sensorSpecificEventStr_EN[31][3] = "Diagnostic Boot Completed"
sensorSpecificEventStr_EN[31][4] = "CD-ROM Boot Completed"
sensorSpecificEventStr_EN[31][5] = "ROM Boot Completed"
sensorSpecificEventStr_EN[31][6] = "Boot Completed - Boot Device Not Specified"
sensorSpecificEventStr_EN[31][7] = "Base OS/Hypervisor Installation started"
sensorSpecificEventStr_EN[31][8] = "Base OS/Hypervisor Installation completed"
sensorSpecificEventStr_EN[31][9] = "Base OS/Hypervisor Installation aborted"
sensorSpecificEventStr_EN[31][10] = "Base OS/Hypervisor Installation failed"

sensorSpecificEventStr_EN[32] = {}
sensorSpecificEventStr_EN[32][0] = "Critical stop during OS load / initialization"
sensorSpecificEventStr_EN[32][1] = "Run-time Critical Stop"
sensorSpecificEventStr_EN[32][2] = "OS Graceful Stop"
sensorSpecificEventStr_EN[32][3] = "OS Graceful Shutdown"
sensorSpecificEventStr_EN[32][4] = "Soft Shutdown initiated by PEF"
sensorSpecificEventStr_EN[32][5] = "Agent Not Responding"

sensorSpecificEventStr_EN[33] = {}
sensorSpecificEventStr_EN[33][0] = "Fault Status Asserted"
sensorSpecificEventStr_EN[33][1] = "Identify Status Asserted"
sensorSpecificEventStr_EN[33][2] = "Slot/Connector Device Installed/Attached"
sensorSpecificEventStr_EN[33][3] = "Slot/Connector Ready For Device Installation"
sensorSpecificEventStr_EN[33][4] = "Slot/Connector Ready For Device Removal"
sensorSpecificEventStr_EN[33][5] = "Slot Power is Off"
sensorSpecificEventStr_EN[33][6] = "Slot/Connector Device Removal Request"
sensorSpecificEventStr_EN[33][7] = "Interlock Asserted"
sensorSpecificEventStr_EN[33][8] = "Slot is Disabled"
sensorSpecificEventStr_EN[33][9] = "Slot Holds Spare Device"

sensorSpecificEventStr_EN[34] = {}
sensorSpecificEventStr_EN[34][0] = "S0/G0 'Working"
sensorSpecificEventStr_EN[34][1] = "S1 'Sleeping With System HW & Processor Context Maintained'"
sensorSpecificEventStr_EN[34][2] = "S2 'Sleeping, Processor Context Lost'"
sensorSpecificEventStr_EN[34][3] = "S3 'Sleeping, Processor & HW Context Lost, Memory Retained'"
sensorSpecificEventStr_EN[34][4] = "S4 'Non-Volatile Sleep/Suspend To Disk'"
sensorSpecificEventStr_EN[34][5] = "S5/G2 'Soft Off'"
sensorSpecificEventStr_EN[34][6] = "S4/S5 Soft Off, Particular S4/S5 State Cannot Be Determined"
sensorSpecificEventStr_EN[34][7] = "G3 / Mechanical Off"
sensorSpecificEventStr_EN[34][8] = "Sleeping in S1, S2, or S3"
sensorSpecificEventStr_EN[34][9] = "G1 Sleeping"
sensorSpecificEventStr_EN[34][10] = "S5 Entered By Override"
sensorSpecificEventStr_EN[34][11] = "Legacy ON State"
sensorSpecificEventStr_EN[34][12] = "Legacy OFF State"
sensorSpecificEventStr_EN[34][13] = "Unknown"

sensorSpecificEventStr_EN[35] = {}
sensorSpecificEventStr_EN[35][0] = "Timer Expired"
sensorSpecificEventStr_EN[35][1] = "Hard Reset"
sensorSpecificEventStr_EN[35][2] = "Power Down"
sensorSpecificEventStr_EN[35][3] = "Power Cycle"
sensorSpecificEventStr_EN[35][4] = "Reserved"
sensorSpecificEventStr_EN[35][5] = "Reserved"
sensorSpecificEventStr_EN[35][6] = "Reserved"
sensorSpecificEventStr_EN[35][7] = "Reserved"
sensorSpecificEventStr_EN[35][8] = "Timer Interrupt"

sensorSpecificEventStr_EN[36] = {}
sensorSpecificEventStr_EN[36][0] = "Platform Generated Page"
sensorSpecificEventStr_EN[36][1] = "Platform Generated LAN Alert"
sensorSpecificEventStr_EN[36][2] = "Platform Event Trap Generated"
sensorSpecificEventStr_EN[36][3] = "Platform Generated SNMP Trap, OEM Format"

sensorSpecificEventStr_EN[37] = {}
sensorSpecificEventStr_EN[37][0] = "Entity Present"
sensorSpecificEventStr_EN[37][1] = "Entity Absent"
sensorSpecificEventStr_EN[37][2] = "Entity Disabled"

sensorSpecificEventStr_EN[39] = {}
sensorSpecificEventStr_EN[39][0] = "LAN Heartbeat Lost"
sensorSpecificEventStr_EN[39][1] = "LAN Heartbeat"

sensorSpecificEventStr_EN[40] = {}
sensorSpecificEventStr_EN[40][0] = "Sensor Access Degraded or Unavailable"
sensorSpecificEventStr_EN[40][1] = "Controller Access Degraded or Unavailable"
sensorSpecificEventStr_EN[40][2] = "Management Controller Off-line"
sensorSpecificEventStr_EN[40][3] = "Management Controller Unavailable"
sensorSpecificEventStr_EN[40][4] = "Sensor failure"
sensorSpecificEventStr_EN[40][5] = "FRU failure"

sensorSpecificEventStr_EN[41] = {}
sensorSpecificEventStr_EN[41][0] = "Battery Low"
sensorSpecificEventStr_EN[41][1] = "Battery Failed"
sensorSpecificEventStr_EN[41][2] = "Battery Presence Detected"

sensorSpecificEventStr_EN[42] = {}
sensorSpecificEventStr_EN[42][0] = "Session Activated"
sensorSpecificEventStr_EN[42][1] = "Session Deactivated"
sensorSpecificEventStr_EN[42][2] = "Invalid Username or Password"
sensorSpecificEventStr_EN[42][3] = "Invalid password disable"

sensorSpecificEventStr_EN[43] = {}
sensorSpecificEventStr_EN[43][0] = "Hardware Change Detected"
sensorSpecificEventStr_EN[43][1] = "Firmware / Software Change Detected"
sensorSpecificEventStr_EN[43][2] = "Hardware Incompatibility Detected"
sensorSpecificEventStr_EN[43][3] = "Firmware / Software Incompatibility Detected"
sensorSpecificEventStr_EN[43][4] = "Invalid or Unsupported Hardware Version"
sensorSpecificEventStr_EN[43][5] = "Invalid or Unsupported Firmware / Software Version"
sensorSpecificEventStr_EN[43][6] = "Hardware Change Detected was successful"
sensorSpecificEventStr_EN[43][7] = "Firmware / Software Change Detected was successful"

sensorSpecificEventStr_EN[44] = {}
sensorSpecificEventStr_EN[44][0] = "FRU Not Installed"
sensorSpecificEventStr_EN[44][1] = "FRU Inactive"
sensorSpecificEventStr_EN[44][2] = "FRU Activation Requested"
sensorSpecificEventStr_EN[44][3] = "FRU Activation In Progress"
sensorSpecificEventStr_EN[44][4] = "FRU Active"
sensorSpecificEventStr_EN[44][5] = "FRU Deactivation Requested"
sensorSpecificEventStr_EN[44][6] = "FRU Deactivation In Progress"
sensorSpecificEventStr_EN[44][7] = "FRU Communication Lost"

sensorSpecificEventStr_EN[194] = {}
sensorSpecificEventStr_EN[194][0] = "PMBus Power Module 0 undefined Error"
sensorSpecificEventStr_EN[194][1] = "PMBus Power Module 0 communicate, memory, controller Error"
sensorSpecificEventStr_EN[194][2] = "PMBus Power Module 0 temperature Error"
sensorSpecificEventStr_EN[194][3] = "PMBus Power Module 0 voltage or current input Error"
sensorSpecificEventStr_EN[194][4] = "PMBus Power Module 0 current output Error"
sensorSpecificEventStr_EN[194][5] = "PMBus Power Module 0 voltage output Error"
sensorSpecificEventStr_EN[194][6] = "PMBus Power Module 0 is off"
sensorSpecificEventStr_EN[194][7] = "PMBus Power Module 0 is busy"
sensorSpecificEventStr_EN[194][8] = "PMBus Power Module 0 unknown Error or Warning"
sensorSpecificEventStr_EN[194][9] = "PMBus Power Module 0 other Error"
sensorSpecificEventStr_EN[194][10] = "PMBus Power Module 0 fan Error"
sensorSpecificEventStr_EN[194][11] = "PMBus Power Module 0 power supply abnormal"
sensorSpecificEventStr_EN[194][12] = "PMBus Power Module 0 user defined Error"
sensorSpecificEventStr_EN[194][13] = "PMBus Power Module 0 voltage or current input Error"
sensorSpecificEventStr_EN[194][14] = "PMBus Power Module 0 current or Power output Error"
sensorSpecificEventStr_EN[194][15] = "PMBus Power Module 0 voltage output Error"

sensorSpecificEventStr_EN[194][16] = "PMBus Power Module 1 undefined Error"
sensorSpecificEventStr_EN[194][17] = "PMBus Power Module 1 communicate, memory, controller Error"
sensorSpecificEventStr_EN[194][18] = "PMBus Power Module 1 temperature Error"
sensorSpecificEventStr_EN[194][19] = "PMBus Power Module 1 voltage or current input Error"
sensorSpecificEventStr_EN[194][20] = "PMBus Power Module 1 current output Error"
sensorSpecificEventStr_EN[194][21] = "PMBus Power Module 1 voltage output Error"
sensorSpecificEventStr_EN[194][22] = "PMBus Power Module 1 is off"
sensorSpecificEventStr_EN[194][23] = "PMBus Power Module 1 is busy"
sensorSpecificEventStr_EN[194][24] = "PMBus Power Module 1 unknown Error or Warning"
sensorSpecificEventStr_EN[194][25] = "PMBus Power Module 1 other Error"
sensorSpecificEventStr_EN[194][26] = "PMBus Power Module 1 fan Error"
sensorSpecificEventStr_EN[194][27] = "PMBus Power Module 1 power supply abnormal"
sensorSpecificEventStr_EN[194][28] = "PMBus Power Module 1 user defined Error"
sensorSpecificEventStr_EN[194][29] = "PMBus Power Module 1 voltage or current input Error"
sensorSpecificEventStr_EN[194][30] = "PMBus Power Module 1 current or Power output Error"
sensorSpecificEventStr_EN[194][31] = "PMBus Power Module 1 voltage output Error"

sensorSpecificEventStr_EN[194][32] = "PMBus Power Module 2 undefined Error"
sensorSpecificEventStr_EN[194][33] = "PMBus Power Module 2 communicate, memory, controller Error"
sensorSpecificEventStr_EN[194][34] = "PMBus Power Module 2 temperature Error"
sensorSpecificEventStr_EN[194][35] = "PMBus Power Module 2 voltage or current input Error"
sensorSpecificEventStr_EN[194][36] = "PMBus Power Module 2 current output Error"
sensorSpecificEventStr_EN[194][37] = "PMBus Power Module 2 voltage output Error"
sensorSpecificEventStr_EN[194][38] = "PMBus Power Module 2 is off"
sensorSpecificEventStr_EN[194][39] = "PMBus Power Module 2 is busy"
sensorSpecificEventStr_EN[194][40] = "PMBus Power Module 2 unknown Error or Warning"
sensorSpecificEventStr_EN[194][41] = "PMBus Power Module 2 other Error"
sensorSpecificEventStr_EN[194][42] = "PMBus Power Module 2 fan Error"
sensorSpecificEventStr_EN[194][43] = "PMBus Power Module 2 power supply abnormal"
sensorSpecificEventStr_EN[194][44] = "PMBus Power Module 2 user defined Error"
sensorSpecificEventStr_EN[194][45] = "PMBus Power Module 2 voltage or current input Error"
sensorSpecificEventStr_EN[194][46] = "PMBus Power Module 2 current or Power output Error"
sensorSpecificEventStr_EN[194][47] = "PMBus Power Module 2 voltage output Error"

sensorSpecificEventStr_EN[194][48] = "PMBus Power Module 3 undefined Error"
sensorSpecificEventStr_EN[194][49] = "PMBus Power Module 3 communicate, memory, controller Error"
sensorSpecificEventStr_EN[194][50] = "PMBus Power Module 3 temperature Error"
sensorSpecificEventStr_EN[194][51] = "PMBus Power Module 3 voltage or current input Error"
sensorSpecificEventStr_EN[194][52] = "PMBus Power Module 3 current output Error"
sensorSpecificEventStr_EN[194][53] = "PMBus Power Module 3 voltage output Error"
sensorSpecificEventStr_EN[194][54] = "PMBus Power Module 3 is off"
sensorSpecificEventStr_EN[194][55] = "PMBus Power Module 3 is busy"
sensorSpecificEventStr_EN[194][56] = "PMBus Power Module 3 unknown Error or Warning"
sensorSpecificEventStr_EN[194][57] = "PMBus Power Module 3 other Error"
sensorSpecificEventStr_EN[194][58] = "PMBus Power Module 3 fan Error"
sensorSpecificEventStr_EN[194][59] = "PMBus Power Module 3 power supply abnormal"
sensorSpecificEventStr_EN[194][60] = "PMBus Power Module 3 user defined Error"
sensorSpecificEventStr_EN[194][61] = "PMBus Power Module 3 voltage or current input Error"
sensorSpecificEventStr_EN[194][62] = "PMBus Power Module 3 current or Power output Error"
sensorSpecificEventStr_EN[194][63] = "PMBus Power Module 3 voltage output Error"

sensorSpecificEventStr_EN[195] = {}
sensorSpecificEventStr_EN[195][0] = "CPU0 Bus and interconnect Error"
sensorSpecificEventStr_EN[195][1] = "CPU0 L3 Tag Error"
sensorSpecificEventStr_EN[195][2] = "CPU0 Internal Error"

sensorSpecificEventStr_EN[195][16] = "CPU1 Bus and interconnect Error"
sensorSpecificEventStr_EN[195][17] = "CPU1 L3 Tag Error"
sensorSpecificEventStr_EN[195][18] = "CPU1 Internal Error"

sensorSpecificEventStr_EN[195][32] = "CPU2 Bus and interconnect Error"
sensorSpecificEventStr_EN[195][33] = "CPU2 L3 Tag Error"
sensorSpecificEventStr_EN[195][34] = "CPU2 Internal Error"

sensorSpecificEventStr_EN[195][48] = "CPU3 Bus and interconnect Error"
sensorSpecificEventStr_EN[195][49] = "CPU3 L3 Tag Error"
sensorSpecificEventStr_EN[195][50] = "CPU3 Internal Error"

sensorSpecificEventStr_EN[196] = {}
sensorSpecificEventStr_EN[196][0] = "CPU0 Channel0 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][1] = "CPU0 Channel0 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][2] = "CPU0 Channel0 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][4] = "CPU0 Channel1 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][5] = "CPU0 Channel1 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][6] = "CPU0 Channel1 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][8] = "CPU0 Channel2 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][9] = "CPU0 Channel2 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][10] = "CPU0 Channel2 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][12] = "CPU0 Channel3 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][13] = "CPU0 Channel3 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][14] = "CPU0 Channel3 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][16] = "CPU1 Channel0 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][17] = "CPU1 Channel0 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][18] = "CPU1 Channel0 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][20] = "CPU1 Channel1 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][21] = "CPU1 Channel1 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][22] = "CPU1 Channel1 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][24] = "CPU1 Channel2 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][25] = "CPU1 Channel2 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][26] = "CPU1 Channel2 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][28] = "CPU1 Channel3 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][29] = "CPU1 Channel3 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][30] = "CPU1 Channel3 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][32] = "CPU2 Channel0 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][33] = "CPU2 Channel0 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][34] = "CPU2 Channel0 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][36] = "CPU2 Channel1 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][37] = "CPU2 Channel1 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][38] = "CPU2 Channel1 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][40] = "CPU2 Channel2 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][41] = "CPU2 Channel2 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][42] = "CPU2 Channel2 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][44] = "CPU2 Channel3 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][45] = "CPU2 Channel3 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][46] = "CPU2 Channel3 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][48] = "CPU3 Channel0 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][49] = "CPU3 Channel0 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][50] = "CPU3 Channel0 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][52] = "CPU3 Channel1 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][53] = "CPU3 Channel1 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][54] = "CPU3 Channel1 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][56] = "CPU3 Channel2 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][57] = "CPU3 Channel2 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][58] = "CPU3 Channel2 Dimm2 memory Error"

sensorSpecificEventStr_EN[196][60] = "CPU3 Channel3 Dimm0 memory Error"
sensorSpecificEventStr_EN[196][61] = "CPU3 Channel3 Dimm1 memory Error"
sensorSpecificEventStr_EN[196][62] = "CPU3 Channel3 Dimm2 memory Error"

sensorSpecificEventStr_EN[197] = {}
sensorSpecificEventStr_EN[197][0] = "PMBus Power Module 1 OV Error"
sensorSpecificEventStr_EN[197][16] = "PMBus Power Module 2 OV Error"

sensorSpecificEventStr_EN[199] = {}
sensorSpecificEventStr_EN[199][0] = "Disk 1  Error"
sensorSpecificEventStr_EN[199][16] = "Disk 2 Error"
sensorSpecificEventStr_EN[199][32] = "Disk 3 Error"
sensorSpecificEventStr_EN[199][48] = "Disk 4 Error"
sensorSpecificEventStr_EN[199][64] = "Disk 5 Error"
sensorSpecificEventStr_EN[199][80] = "Disk 6 Error"

sensorSpecificEventStr_EN[200] = {}
sensorSpecificEventStr_EN[200][0] = "Redundant Test OK"
sensorSpecificEventStr_EN[200][1] = "Redundant Test Not Ready, Input Mode Error"
sensorSpecificEventStr_EN[200][2] = "Redundant Test Fail, Status Word Error"
sensorSpecificEventStr_EN[200][3] = "Input Was Set To AC Forcedly"
sensorSpecificEventStr_EN[200][4] = "Input Was Set To DC Forcedly"
sensorSpecificEventStr_EN[200][5] = "Redundant Test Fail,Input Mode Forcedly Set Fail"
sensorSpecificEventStr_EN[200][6] = "Redundant Test Fail, Cold Redundant Mode Error"
sensorSpecificEventStr_EN[200][7] = "Redundant Test Fail, Send CMD Failed"

sensorSpecificEventStr_EN[201] = {}
sensorSpecificEventStr_EN[201][0] = "BIOS Setup Options Cleared in BMC"
sensorSpecificEventStr_EN[201][1] = "BIOS Setup Options Setted in BMC"

sensorSpecificEventStr_CN = {}
sensorSpecificEventStr_CN[0] = "保留"
sensorSpecificEventStr_CN[1] = "温度"
sensorSpecificEventStr_CN[2] = "电压"
sensorSpecificEventStr_CN[3] = "电流"
sensorSpecificEventStr_CN[4] = "风扇"
sensorSpecificEventStr_CN[5] = {}
sensorSpecificEventStr_CN[5][0] = "一般机箱入侵"
sensorSpecificEventStr_CN[5][1] = "驱动器托架入侵"
sensorSpecificEventStr_CN[5][2] = "I/O卡区入侵"
sensorSpecificEventStr_CN[5][3] = "处理器区域入侵"
sensorSpecificEventStr_CN[5][4] = "局域网丢失（系统从局域网拔除）"
sensorSpecificEventStr_CN[5][5] = "未受权的插入/移除"
sensorSpecificEventStr_CN[5][6] = "风扇区域入侵"
sensorSpecificEventStr_CN[6] = {}
sensorSpecificEventStr_CN[6][0] = "安全模式违反企图"
sensorSpecificEventStr_CN[6][1] = "违反前开机密码-用户密码"
sensorSpecificEventStr_CN[6][2] = "违反前开机密码-设置密码"
sensorSpecificEventStr_CN[6][3] = "违反前开机密码-网络引导密码"
sensorSpecificEventStr_CN[6][4] = "其他违反前开机密码"
sensorSpecificEventStr_CN[6][5] = "违反带外访问密码"
sensorSpecificEventStr_CN[7] = {}
sensorSpecificEventStr_CN[7][0] = "IERR"
sensorSpecificEventStr_CN[7][1] = "过热保护"
sensorSpecificEventStr_CN[7][2] = "FRB1/BIST故障"
sensorSpecificEventStr_CN[7][3] = "FRB2/开机阶段宕机故障"
sensorSpecificEventStr_CN[7][4] = "FRB3/处理器初始化故障"
sensorSpecificEventStr_CN[7][5] = "配置故障（DMI）"
sensorSpecificEventStr_CN[7][6] = "SMBIOS无法纠正的CPU复杂故障"
sensorSpecificEventStr_CN[7][7] = "检测到处理器的存在"
sensorSpecificEventStr_CN[7][8] = "处理器禁用"
sensorSpecificEventStr_CN[7][9] = "检测到终端的存在"
sensorSpecificEventStr_CN[7][10] = "处理器自动节流"
sensorSpecificEventStr_CN[7][11] = "MCE错误"
sensorSpecificEventStr_CN[7][12] = "可恢复的MCE错误"

sensorSpecificEventStr_CN[8] = {}
sensorSpecificEventStr_CN[8][0] = "检测到存在"
sensorSpecificEventStr_CN[8][1] = "检测到电源故障"
sensorSpecificEventStr_CN[8][2] = "预测性故障生效"
sensorSpecificEventStr_CN[8][3] = "电源输入丢失（交流/直流）"
sensorSpecificEventStr_CN[8][4] = "电源输入丢失或者超出范围"
sensorSpecificEventStr_CN[8][5] = "电源输入超出范围，但是存在。"
sensorSpecificEventStr_CN[8][6] = "配置故障"
sensorSpecificEventStr_CN[8][7] = "PSU电源输入丢失"
sensorSpecificEventStr_CN[8][8] = "PCH电源输入丢失"
sensorSpecificEventStr_CN[8][9] = "PSU B0模块不存在"
sensorSpecificEventStr_CN[8][10] = "PSU B2模块不存在"
sensorSpecificEventStr_CN[8][11] = "PSU B4模块不存在"
sensorSpecificEventStr_CN[8][12] = "PSU B6模块不存在"
sensorSpecificEventStr_CN[9] = {}
sensorSpecificEventStr_CN[9][0] = "关机"
sensorSpecificEventStr_CN[9][1] = "循环开关机"
sensorSpecificEventStr_CN[9][2] = "240VA断电"
sensorSpecificEventStr_CN[9][3] = "连锁断电"
sensorSpecificEventStr_CN[9][4] = "交流丢失"
sensorSpecificEventStr_CN[9][5] = "软开关机控制失败"
sensorSpecificEventStr_CN[9][6] = "检测到电源单元故障"
sensorSpecificEventStr_CN[9][7] = "预测故障"
sensorSpecificEventStr_CN[12] = {}
sensorSpecificEventStr_CN[12][0] = "可更正的ECC故障"
sensorSpecificEventStr_CN[12][1] = "不可更正的ECC故障"
sensorSpecificEventStr_CN[12][2] = "奇偶校验"
sensorSpecificEventStr_CN[12][3] = "内存擦写故障"
sensorSpecificEventStr_CN[12][4] = "内存设备禁用"
sensorSpecificEventStr_CN[12][5] = "可更正的ECC日志记录到达限制"
sensorSpecificEventStr_CN[12][6] = "检测到存在"
sensorSpecificEventStr_CN[12][7] = "配置故障"
sensorSpecificEventStr_CN[12][8] = "空"
sensorSpecificEventStr_CN[12][9] = "节流"
sensorSpecificEventStr_CN[12][15] = "自定义故障"
sensorSpecificEventStr_CN[13] = {}
sensorSpecificEventStr_CN[13][0] = "硬盘存在"
sensorSpecificEventStr_CN[13][1] = "硬盘错误"
sensorSpecificEventStr_CN[13][2] = "故障预测"
sensorSpecificEventStr_CN[13][3] = "热备用"
sensorSpecificEventStr_CN[13][4] = "一致性检查/奇偶检查处理"
sensorSpecificEventStr_CN[13][5] = "在关键组"
sensorSpecificEventStr_CN[13][6] = "在故障组"
sensorSpecificEventStr_CN[13][7] = "重建/映射处理中"
sensorSpecificEventStr_CN[13][8] = "重建/映射失败"
sensorSpecificEventStr_CN[15] = {}
sensorSpecificEventStr_CN[15][0] = "故障"
sensorSpecificEventStr_CN[15][1] = "宕机"
sensorSpecificEventStr_CN[15][2] = "处理"
sensorSpecificEventStr_CN[16] = {}
sensorSpecificEventStr_CN[16][0] = "可更正内存故障日志记录禁用"
sensorSpecificEventStr_CN[16][1] = "事件类型日志记录禁用"
sensorSpecificEventStr_CN[16][2] = "日志记录区域复位/清空"
sensorSpecificEventStr_CN[16][3] = "所有事件日志记录禁用"
sensorSpecificEventStr_CN[16][4] = "SEL满"
sensorSpecificEventStr_CN[16][5] = "SEL将近满"
sensorSpecificEventStr_CN[17] = {}
sensorSpecificEventStr_CN[17][0] = "BIOS看门狗复位"
sensorSpecificEventStr_CN[17][1] = "操作系统看门狗复位"
sensorSpecificEventStr_CN[17][2] = "操作系统看门狗关机"
sensorSpecificEventStr_CN[17][3] = "操作系统看门狗关机"
sensorSpecificEventStr_CN[17][4] = "操作系统看门狗循环开关机"
sensorSpecificEventStr_CN[17][5] = "操作系统看门狗NMI"
sensorSpecificEventStr_CN[17][6] = "操作系统看门狗计数完成"
sensorSpecificEventStr_CN[17][7] = "OS看门狗预超时中断，非NMI"
sensorSpecificEventStr_CN[18] = {}
sensorSpecificEventStr_CN[18][0] = "系统重配置"
sensorSpecificEventStr_CN[18][1] = "OEM系统启动事件"
sensorSpecificEventStr_CN[18][2] = "未定义的系统硬件故障"
sensorSpecificEventStr_CN[18][3] = "条目添加到辅助日志"
sensorSpecificEventStr_CN[18][4] = "PEF动作"
sensorSpecificEventStr_CN[18][5] = "SEL时钟同步(OS)"
sensorSpecificEventStr_CN[19] = {}
sensorSpecificEventStr_CN[19][0] = "前面板NMI"
sensorSpecificEventStr_CN[19][1] = "总线超时"
sensorSpecificEventStr_CN[19][2] = "I/O通道检测NMI"
sensorSpecificEventStr_CN[19][3] = "软件NMI"
sensorSpecificEventStr_CN[19][4] = "PCI PERR"
sensorSpecificEventStr_CN[19][5] = "PCI SERR"
sensorSpecificEventStr_CN[19][6] = "EISA安全超时失败"
sensorSpecificEventStr_CN[19][7] = "总线可更正故障"
sensorSpecificEventStr_CN[19][8] = "总线不可更正故障"
sensorSpecificEventStr_CN[19][9] = "严重的NMI"
sensorSpecificEventStr_CN[19][10] = "严重的总线故障"
sensorSpecificEventStr_CN[19][11] = "检测到严重的SMLINK0故障"
sensorSpecificEventStr_CN[19][12] = "检测到严重的SMLINK1故障"
sensorSpecificEventStr_CN[20] = {}
sensorSpecificEventStr_CN[20][0] = "开/关键按下"
sensorSpecificEventStr_CN[20][1] = "睡眠键按下"
sensorSpecificEventStr_CN[20][2] = "复位键按下 或者 系统重启动"
sensorSpecificEventStr_CN[20][3] = "FRU锁打开"
sensorSpecificEventStr_CN[20][4] = "FRU服务请求键"
sensorSpecificEventStr_CN[22] = {}
sensorSpecificEventStr_CN[22][0] = "检测到CPU故障"
sensorSpecificEventStr_CN[25] = {}
sensorSpecificEventStr_CN[25][0] = "软开关机控制失败"
sensorSpecificEventStr_CN[25][1] = "过热保护"
sensorSpecificEventStr_CN[27] = {}
sensorSpecificEventStr_CN[27][0] = "电缆/网络已连接"
sensorSpecificEventStr_CN[27][1] = "故障"
sensorSpecificEventStr_CN[27][2] = "未连接"
sensorSpecificEventStr_CN[29] = {}
sensorSpecificEventStr_CN[29][0] = "开始通电"
sensorSpecificEventStr_CN[29][1] = "开始硬件复位"
sensorSpecificEventStr_CN[29][2] = "开始热复位"
sensorSpecificEventStr_CN[29][3] = "用户请求PXE启动"
sensorSpecificEventStr_CN[29][4] = "自动引导到诊断"
sensorSpecificEventStr_CN[29][5] = "OS/运行时软件启动硬复位"
sensorSpecificEventStr_CN[29][6] = "OS/运行时软件启动热复位"
sensorSpecificEventStr_CN[29][7] = "系统重启动"
sensorSpecificEventStr_CN[30] = {}
sensorSpecificEventStr_CN[30][0] = "没有可引导介质"
sensorSpecificEventStr_CN[30][1] = "非引导软盘留在驱动器"
sensorSpecificEventStr_CN[30][2] = "未发现PXE服务器"
sensorSpecificEventStr_CN[30][3] = "无效的引导扇区"
sensorSpecificEventStr_CN[30][4] = "等待用户选择启动资源超时"
sensorSpecificEventStr_CN[31] = {}
sensorSpecificEventStr_CN[31][0] = "A:启动完成"
sensorSpecificEventStr_CN[31][1] = "C:启动完成"
sensorSpecificEventStr_CN[31][2] = "PXE启动完成"
sensorSpecificEventStr_CN[31][3] = "诊断引导完成"
sensorSpecificEventStr_CN[31][4] = "CD-ROM引导完成"
sensorSpecificEventStr_CN[31][5] = "ROM引导完成"
sensorSpecificEventStr_CN[31][6] = "引导完成-目标操作系统未知"
sensorSpecificEventStr_CN[31][7] = "基本OS/虚拟机开始安装"
sensorSpecificEventStr_CN[31][8] = "基本OS/虚拟机安装完成"
sensorSpecificEventStr_CN[31][9] = "基本OS/虚拟机安装中止"
sensorSpecificEventStr_CN[31][10] = "基本OS/虚拟机安装失败"
sensorSpecificEventStr_CN[32] = {}
sensorSpecificEventStr_CN[32][0] = "操作系统初始化过程中停止"
sensorSpecificEventStr_CN[32][1] = "运行时停止"
sensorSpecificEventStr_CN[32][2] = "OS正常停止"
sensorSpecificEventStr_CN[32][3] = "OS正常关机"
sensorSpecificEventStr_CN[32][4] = "由PEF触发的软关机"
sensorSpecificEventStr_CN[32][5] = "代理无响应"
sensorSpecificEventStr_CN[33] = {}
sensorSpecificEventStr_CN[33][0] = "故障状态生效"
sensorSpecificEventStr_CN[33][1] = "确定状态生效"
sensorSpecificEventStr_CN[33][2] = "槽/连接器设备已连接"
sensorSpecificEventStr_CN[33][3] = "槽/连接器准备安装"
sensorSpecificEventStr_CN[33][4] = "槽/连接器准备移除"
sensorSpecificEventStr_CN[33][5] = "插槽的电源是关闭"
sensorSpecificEventStr_CN[33][6] = "槽/连接器移除请求"
sensorSpecificEventStr_CN[33][7] = "连锁生效"
sensorSpecificEventStr_CN[33][8] = "槽禁用"
sensorSpecificEventStr_CN[33][9] = "槽可以容纳备用设备"
sensorSpecificEventStr_CN[34] = {}
sensorSpecificEventStr_CN[34][0] = "S0/G0 “运行中”"
sensorSpecificEventStr_CN[34][1] = "S1 “睡眠，系统硬件及处理器保留”"
sensorSpecificEventStr_CN[34][2] = "S2 “睡眠，处理器丢失”"
sensorSpecificEventStr_CN[34][3] = "S3 “睡眠，处理器和硬件丢失，内存保留”"
sensorSpecificEventStr_CN[34][4] = "S4 “睡眠/挂起”"
sensorSpecificEventStr_CN[34][5] = "S5/G2 “软关机”"
sensorSpecificEventStr_CN[34][6] = "S4/S5 “软关机，需要指出的是S4/S5状态不确定”"
sensorSpecificEventStr_CN[34][7] = "G3 “机械关机”"
sensorSpecificEventStr_CN[34][8] = "睡眠在S1, S2,或者S3"
sensorSpecificEventStr_CN[34][9] = "G1睡眠"
sensorSpecificEventStr_CN[34][10] = "进入到S5"
sensorSpecificEventStr_CN[34][11] = "停留在开机状态"
sensorSpecificEventStr_CN[34][12] = "停留在关机状态"
sensorSpecificEventStr_CN[34][13] = "未知"
sensorSpecificEventStr_CN[35] = {}
sensorSpecificEventStr_CN[35][0] = "时间到"
sensorSpecificEventStr_CN[35][1] = "硬件重启动"
sensorSpecificEventStr_CN[35][2] = "关机"
sensorSpecificEventStr_CN[35][3] = "循环开关机"
sensorSpecificEventStr_CN[35][4] = "保留"
sensorSpecificEventStr_CN[35][5] = "保留"
sensorSpecificEventStr_CN[35][6] = "保留"
sensorSpecificEventStr_CN[35][7] = "保留"
sensorSpecificEventStr_CN[35][8] = "时钟中断"
sensorSpecificEventStr_CN[36] = {}
sensorSpecificEventStr_CN[36][0] = "平台生成的页面"
sensorSpecificEventStr_CN[36][1] = "平台生成的LAN告警"
sensorSpecificEventStr_CN[36][2] = "生成平台事件Trap"
sensorSpecificEventStr_CN[36][3] = "生成平台SNMP Trap，OEM格式"
sensorSpecificEventStr_CN[37] = {}
sensorSpecificEventStr_CN[37][0] = "实体存在"
sensorSpecificEventStr_CN[37][1] = "实体缺席"
sensorSpecificEventStr_CN[37][2] = "实体禁用"
sensorSpecificEventStr_CN[39] = {}
sensorSpecificEventStr_CN[39][0] = "局域网心跳丢失"
sensorSpecificEventStr_CN[39][1] = "局域网心跳"
sensorSpecificEventStr_CN[40] = {}
sensorSpecificEventStr_CN[40][0] = "传感器无法访问或不可用"
sensorSpecificEventStr_CN[40][1] = "控制器无法访问或不可用"
sensorSpecificEventStr_CN[40][2] = "管理控制器下线"
sensorSpecificEventStr_CN[40][3] = "管理控制器不可用"
sensorSpecificEventStr_CN[40][4] = "传感器故障"
sensorSpecificEventStr_CN[40][5] = "FRU故障"
sensorSpecificEventStr_CN[41] = {}
sensorSpecificEventStr_CN[41][0] = "电池电量低"
sensorSpecificEventStr_CN[41][1] = "电池故障"
sensorSpecificEventStr_CN[41][2] = "检测到电池的存在"
sensorSpecificEventStr_CN[42] = {}
sensorSpecificEventStr_CN[42][0] = "会话激活"
sensorSpecificEventStr_CN[42][1] = "会话未激活"
sensorSpecificEventStr_CN[43] = {}
sensorSpecificEventStr_CN[43][0] = "检测到硬件变更"
sensorSpecificEventStr_CN[43][1] = "检测到固件/软件变更"
sensorSpecificEventStr_CN[43][2] = "检测到硬件不兼容"
sensorSpecificEventStr_CN[43][3] = "检测到固件/软件不兼容"
sensorSpecificEventStr_CN[43][4] = "无效或者不支持的硬件版本"
sensorSpecificEventStr_CN[43][5] = "无效或者不支持的固件/软件版本"
sensorSpecificEventStr_CN[43][6] = "检测到硬件的变化是成功的"
sensorSpecificEventStr_CN[43][7] = "检测到固件/软件的变化是成功的"
sensorSpecificEventStr_CN[44] = {}
sensorSpecificEventStr_CN[44][0] = "FRU未安装"
sensorSpecificEventStr_CN[44][1] = "FRU无效"
sensorSpecificEventStr_CN[44][2] = "要求的FRU启用"
sensorSpecificEventStr_CN[44][3] = "FRU可用"
sensorSpecificEventStr_CN[44][4] = "FRU活动"
sensorSpecificEventStr_CN[44][5] = "要求的FRU停用"
sensorSpecificEventStr_CN[44][6] = "FRU可用"
sensorSpecificEventStr_CN[44][7] = "FRU通讯丢失"
sensorSpecificEventStr_CN[194] = {}
sensorSpecificEventStr_CN[194][0] = "PMBus电源模块0出现未定义错误"
sensorSpecificEventStr_CN[194][1] = "PMBus电源模块0通讯，内存或者控制器故障"
sensorSpecificEventStr_CN[194][2] = "PMBus电源模块0温度故障"
sensorSpecificEventStr_CN[194][3] = "PMBus电源模块0输入电压或电流错误"
sensorSpecificEventStr_CN[194][4] = "PMBus电源模块0输出电流故障"
sensorSpecificEventStr_CN[194][5] = "PMBus电源模块0输出电压故障"
sensorSpecificEventStr_CN[194][6] = "PMBus电源模块0不在线"
sensorSpecificEventStr_CN[194][7] = "PMBus电源模块0正在忙"
sensorSpecificEventStr_CN[194][8] = "PMBus电源模块0未知的故障或告警"
sensorSpecificEventStr_CN[194][9] = "PMBus电源模块0其他故障"
sensorSpecificEventStr_CN[194][10] = "PMBus电源模块0风扇故障"
sensorSpecificEventStr_CN[194][11] = "PMBus电源模块0电源供电异常"
sensorSpecificEventStr_CN[194][12] = "PMBus电源模块0用户自定义故障"
sensorSpecificEventStr_CN[194][13] = "PMBus电源模块0输入电压或电流故障"
sensorSpecificEventStr_CN[194][14] = "PMBus电源模块0输出功率或电流故障"
sensorSpecificEventStr_CN[194][15] = "PMBus电源模块0输出电压故障"
sensorSpecificEventStr_CN[194][16] = "PMBus电源模块1出现未定义故障"
sensorSpecificEventStr_CN[194][17] = "PMBus电源模块1通讯，内存或者控制器故障"
sensorSpecificEventStr_CN[194][18] = "PMBus电源模块1温度故障"
sensorSpecificEventStr_CN[194][19] = "PMBus电源模块1输入电压或电流错误"
sensorSpecificEventStr_CN[194][20] = "PMBus电源模块1输出电流故障"
sensorSpecificEventStr_CN[194][21] = "PMBus电源模块1输出电压故障"
sensorSpecificEventStr_CN[194][22] = "PMBus电源模块1不在线"
sensorSpecificEventStr_CN[194][23] = "PMBus电源模块1正在忙"
sensorSpecificEventStr_CN[194][24] = "PMBus电源模块1未知的故障或告警"
sensorSpecificEventStr_CN[194][25] = "PMBus电源模块1其他故障"
sensorSpecificEventStr_CN[194][26] = "PMBus电源模块1风扇故障"
sensorSpecificEventStr_CN[194][27] = "PMBus电源模块1电源供电异常"
sensorSpecificEventStr_CN[194][28] = "PMBus电源模块1用户自定义故障"
sensorSpecificEventStr_CN[194][29] = "PMBus电源模块1输入电压或电流故障"
sensorSpecificEventStr_CN[194][30] = "PMBus电源模块1输出功率或电流故障"
sensorSpecificEventStr_CN[194][31] = "PMBus电源模块1输出电压故障"
sensorSpecificEventStr_CN[194][32] = "PMBus电源模块2出现未定义故障"
sensorSpecificEventStr_CN[194][33] = "PMBus电源模块2通讯，内存或者控制器故障"
sensorSpecificEventStr_CN[194][34] = "PMBus电源模块2温度故障"
sensorSpecificEventStr_CN[194][35] = "PMBus电源模块2输入电压或电流错误"
sensorSpecificEventStr_CN[194][36] = "PMBus电源模块2输出电流故障"
sensorSpecificEventStr_CN[194][37] = "PMBus电源模块2输出电压故障"
sensorSpecificEventStr_CN[194][38] = "PMBus电源模块2不在线"
sensorSpecificEventStr_CN[194][39] = "PMBus电源模块2正在忙"
sensorSpecificEventStr_CN[194][40] = "PMBus电源模块2未知的故障或告警"
sensorSpecificEventStr_CN[194][41] = "PMBus电源模块2其他故障"
sensorSpecificEventStr_CN[194][42] = "PMBus电源模块2风扇故障"
sensorSpecificEventStr_CN[194][43] = "PMBus电源模块2电源供电异常"
sensorSpecificEventStr_CN[194][44] = "PMBus电源模块2用户自定义故障"
sensorSpecificEventStr_CN[194][45] = "PMBus电源模块2输入电压或电流故障"
sensorSpecificEventStr_CN[194][46] = "PMBus电源模块2输出功率或电流故障"
sensorSpecificEventStr_CN[194][47] = "PMBus电源模块2输出电压故障"
sensorSpecificEventStr_CN[194][48] = "PMBus电源模块3出现未定义故障"
sensorSpecificEventStr_CN[194][49] = "PMBus电源模块3通讯，内存或者控制器故障"
sensorSpecificEventStr_CN[194][50] = "PMBus电源模块3温度故障"
sensorSpecificEventStr_CN[194][51] = "PMBus电源模块3输入电压或电流错误"
sensorSpecificEventStr_CN[194][52] = "PMBus电源模块3输出电流故障"
sensorSpecificEventStr_CN[194][53] = "PMBus电源模块3输出电压故障"
sensorSpecificEventStr_CN[194][54] = "PMBus电源模块3不在线"
sensorSpecificEventStr_CN[194][55] = "PMBus电源模块3正在忙"
sensorSpecificEventStr_CN[194][56] = "PMBus电源模块3未知的故障或告警"
sensorSpecificEventStr_CN[194][57] = "PMBus电源模块3其他故障"
sensorSpecificEventStr_CN[194][58] = "PMBus电源模块3风扇故障"
sensorSpecificEventStr_CN[194][59] = "PMBus电源模块3电源供电异常"
sensorSpecificEventStr_CN[194][60] = "PMBus电源模块3用户自定义故障"
sensorSpecificEventStr_CN[194][61] = "PMBus电源模块3输入电压或电流故障"
sensorSpecificEventStr_CN[194][62] = "PMBus电源模块3输出功率或电流故障"
sensorSpecificEventStr_CN[194][63] = "PMBus电源模块3输出电压故障"
sensorSpecificEventStr_CN[195] = {}
sensorSpecificEventStr_CN[195][0] = "CPU0发生Bus和Interconnect故障"
sensorSpecificEventStr_CN[195][1] = "CPU0发生L3 Tag故障"
sensorSpecificEventStr_CN[195][2] = "CPU0发生内部故障"
sensorSpecificEventStr_CN[195][16] = "CPU1发生Bus和Interconnect故障"
sensorSpecificEventStr_CN[195][17] = "CPU1发生L3 Tag故障"
sensorSpecificEventStr_CN[195][18] = "CPU1发生内部故障"
sensorSpecificEventStr_CN[195][32] = "CPU2发生Bus和Interconnect故障"
sensorSpecificEventStr_CN[195][33] = "CPU2发生L3 Tag故障"
sensorSpecificEventStr_CN[195][34] = "CPU2发生内部故障"
sensorSpecificEventStr_CN[195][48] = "CPU3发生Bus和Interconnect故障"
sensorSpecificEventStr_CN[195][49] = "CPU3发生L3 Tag故障"
sensorSpecificEventStr_CN[195][50] = "CPU3发生内部故障"
sensorSpecificEventStr_CN[196] = {}
sensorSpecificEventStr_CN[196][0] = "CPU0,Channel0,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][1] = "CPU0,Channel0,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][2] = "CPU0,Channel0,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][4] = "CPU0,Channel1,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][5] = "CPU0,Channel1,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][6] = "CPU0,Channel1,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][8] = "CPU0,Channel2,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][9] = "CPU0,Channel2,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][10] = "CPU0,Channel2,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][12] = "CPU0,Channel3,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][13] = "CPU0,Channel3,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][14] = "CPU0,Channel3,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][16] = "CPU1,Channel0,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][17] = "CPU1,Channel0,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][18] = "CPU1,Channel0,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][20] = "CPU1,Channel1,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][21] = "CPU1,Channel1,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][22] = "CPU1,Channel1,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][24] = "CPU1,Channel2,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][25] = "CPU1,Channel2,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][26] = "CPU1,Channel2,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][28] = "CPU1,Channel3,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][29] = "CPU1,Channel3,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][30] = "CPU1,Channel3,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][32] = "CPU2,Channel0,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][33] = "CPU2,Channel0,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][34] = "CPU2,Channel0,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][36] = "CPU2,Channel1,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][37] = "CPU2,Channel1,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][38] = "CPU2,Channel1,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][40] = "CPU2,Channel2,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][41] = "CPU2,Channel2,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][42] = "CPU2,Channel2,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][44] = "CPU2,Channel3,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][45] = "CPU2,Channel3,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][46] = "CPU2,Channel3,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][48] = "CPU3,Channel0,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][49] = "CPU3,Channel0,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][50] = "CPU3,Channel0,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][52] = "CPU3,Channel1,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][53] = "CPU3,Channel1,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][54] = "CPU3,Channel1,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][56] = "CPU3,Channel2,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][57] = "CPU3,Channel2,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][58] = "CPU3,Channel2,Dimm2发生内存故障"
sensorSpecificEventStr_CN[196][60] = "CPU3,Channel3,Dimm0发生内存故障"
sensorSpecificEventStr_CN[196][61] = "CPU3,Channel3,Dimm1发生内存故障"
sensorSpecificEventStr_CN[196][62] = "CPU3,Channel3,Dimm2发生内存故障"
sensorSpecificEventStr_CN[197] = {}
sensorSpecificEventStr_CN[197][0] = "PMBus电源模块1电压过载故障"
sensorSpecificEventStr_CN[197][16] = "PMBus电源模块2电压过载故障"

sensorSpecificEventStr_CN[199] = {}
sensorSpecificEventStr_CN[199][0] = "Disk 1  故障"
sensorSpecificEventStr_CN[199][16] = "Disk 2 故障"
sensorSpecificEventStr_CN[199][32] = "Disk 3 故障"
sensorSpecificEventStr_CN[199][48] = "Disk 4 故障"
sensorSpecificEventStr_CN[199][64] = "Disk 5 故障"
sensorSpecificEventStr_CN[199][80] = "Disk 6 故障"

sensorSpecificEventStr_CN[200] = {}
sensorSpecificEventStr_CN[200][0] = "电源冗余测试通过"
sensorSpecificEventStr_CN[200][1] = "电源冗余测试未就绪，输入模式错误"
sensorSpecificEventStr_CN[200][2] = "电源冗余测试失败，状态字异常"
sensorSpecificEventStr_CN[200][3] = "AC电源被设置成输入"
sensorSpecificEventStr_CN[200][4] = "DC电源被设置成输入"
sensorSpecificEventStr_CN[200][5] = "电源冗余测试失败，无法设置电源冗余状态"
sensorSpecificEventStr_CN[200][6] = "电源冗余测试失败，电源当前冗余状态错误"
sensorSpecificEventStr_CN[200][7] = "电源冗余测试失败，发送PMbus命令失败"

sensorSpecificEventStr_CN[201] = {}
sensorSpecificEventStr_CN[201][0] = "BMC中的BIOS配置参数被清空"
sensorSpecificEventStr_CN[201][1] = "BMC中的BIOS配置参数被设置"

sensorSpecificEventStr = {}
sensorSpecificEventStr["en"] = sensorSpecificEventStr_EN
sensorSpecificEventStr["cn"] = sensorSpecificEventStr_CN



# -*- coding:utf-8 -*-

from inmanage_sdk.util import RegularCheckUtil
from inmanage_sdk.command import RestFunc

from inmanage_sdk.interface.CommonM6_41401 import CommonM6_41401
from inmanage_sdk.interface.ResEntity import ResultBean, SnmpBean, DestinationTXBean


class CommonM6_4140a(CommonM6_41401):

    def getalertpolicy(self, client, args):
        alertinfo = ResultBean()
        result = self.gettrap(client, args)
        if result.State == "Success" and len(result.Message) > 0:
            alert_policy = result.Message[0].get('Destination')
            alertinfo.State("Success")
            alertinfo.Message(alert_policy)
        else:
            alertinfo.State("Failure")
            alertinfo.Message(["get alert policy information error"])
        return alertinfo

    def getsnmptrap(self, client, args):
        trapinfo = ResultBean()
        result = self.gettrap(client, args)
        if result.State == "Success" and len(result.Message) > 0:
            snmptrap = result.Message[0]
            del snmptrap['Destination']
            trapinfo.State("Success")
            trapinfo.Message(snmptrap)
        else:
            trapinfo.State("Failure")
            trapinfo.Message(["get snmp trap information error"])
        return trapinfo

    def gettrap(self, client, args):
        # login
        headers = RestFunc.login_M6(client)
        if headers == {}:
            login_res = ResultBean()
            login_res.State("Failure")
            login_res.Message(["login error, please check username/password/host/port"])
            return login_res
        client.setHearder(headers)

        # get
        res = RestFunc.getSnmpInfoByRest(client)
        snmpinfo = ResultBean()
        if res == {}:
            snmpinfo.State("Failure")
            snmpinfo.Message(["cannot get snmp information"])
        elif res.get('code') == 0 and res.get('data') is not None:
            snmpinfo.State("Success")
            version_dict = {1: "V1", 2: "V2", 3: "V3"}
            severity_dict = {0: "All", 1: "WarningAndCritical", 2: "Critical", "info": "All",
                             "warning": "WarningAndCritical", "critical": "Critical"}
            status_dict = {1: "Enabled", 0: "Disabled", "Enable": "Enabled", "Disable": "Disabled"}
            authentication_dict = {0: "NONE", 1: "SHA", 2: "MD5"}
            privacy_dict = {0: "NONE", 1: "DES", 2: "AES"}
            item = res.get('data')
            snmpbean = SnmpBean()
            SnmpTrapCfg = item.get('SnmpTrapCfg')
            # if SnmpTrapCfg['TrapVersion'] == 0:
            if SnmpTrapCfg['TrapVersion'] == "Disable":
                snmpbean.Enable('Disabled')
            else:
                snmpbean.Enable('Enabled')
                # 021 returns key012 022+returns value
                snmpbean.TrapVersion(version_dict.get(SnmpTrapCfg['TrapVersion'], SnmpTrapCfg['TrapVersion']))
                snmpbean.Community(SnmpTrapCfg['Community'])
                snmpbean.Severity(severity_dict.get(SnmpTrapCfg['EventLevelLimit'], SnmpTrapCfg['EventLevelLimit']))
                SnmpTrapDestCfg = item.get('SnmpTrapDestCfg')
                snmpTrapDestList = []
                for std in SnmpTrapDestCfg:
                    stdnew = DestinationTXBean()
                    stdnew.Id(std["id"] + 1)
                    stdnew.Enable(status_dict.get(std["Enabled"], std["Enabled"]))
                    if std["Destination"].strip() == "":
                        stdnew.Address(None)
                    else:
                        stdnew.Address(std["Destination"])
                    stdnew.Port(std["port"])
                    snmpTrapDestList.append(stdnew.dict)
                snmpbean.Destination(snmpTrapDestList)
                snmpbean.AUTHProtocol(authentication_dict.get(SnmpTrapCfg['AUTHProtocol'], SnmpTrapCfg['AUTHProtocol']))
                snmpbean.AUTHPwd(SnmpTrapCfg['AUTHPwd'])
                snmpbean.PRIVProtocol(privacy_dict.get(SnmpTrapCfg['PrivProtocol'], SnmpTrapCfg['PrivProtocol']))
                snmpbean.PRIVPwd(SnmpTrapCfg['PRIVPwd'])
                snmpbean.EngineID(SnmpTrapCfg['EngineID'])
                snmpbean.DeviceType(SnmpTrapCfg['DeviceType'])
                snmpbean.HostID(SnmpTrapCfg['HostID'])
                snmpbean.UserName(SnmpTrapCfg['UserName'])
            snmpinfo.Message([snmpbean.dict])
        elif res.get('code') != 0 and res.get('data') is not None:
            snmpinfo.State("Failure")
            snmpinfo.Message([res.get('data')])
        else:
            snmpinfo.State("Failure")
            snmpinfo.Message(["get snmp information error, error code " + str(res.get('code'))])

        RestFunc.logout(client)
        return snmpinfo

    def setsnmptrap(self, client, args):
        alertinfo = self.settrapcom(client, args)
        return alertinfo

    def settrapcom(self, client, args):
        # login
        headers = RestFunc.login_M6(client)
        if headers == {}:
            login_res = ResultBean()
            login_res.State("Failure")
            login_res.Message(["login error, please check username/password/host/port"])
            return login_res
        client.setHearder(headers)

        editFlag = False
        versionFlag = False
        res = RestFunc.getSnmpInfoByRest(client)
        snmpinfo = ResultBean()
        if res == {}:
            snmpinfo.State("Failure")
            snmpinfo.Message(["cannot get snmp information"])
            RestFunc.logout(client)
            return snmpinfo
        elif res.get('code') == 0 and res.get('data') is not None:
            item = res.get('data')
            SnmpTrapCfg = item.get('SnmpTrapCfg')
            default_trap_version = SnmpTrapCfg.get('TrapVersion')
            default_enent_severity = SnmpTrapCfg.get('EventLevelLimit')
            default_community = SnmpTrapCfg.get('Community', '')
            default_v3username = SnmpTrapCfg.get('UserName')
            default_engine_Id = SnmpTrapCfg.get('EngineID')
            default_auth = SnmpTrapCfg.get('AUTHProtocol')
            default_auth_pass = SnmpTrapCfg.get('AUTHPwd', '')
            default_priv = SnmpTrapCfg.get('PrivProtocol')
            default_priv_pass = SnmpTrapCfg.get('PRIVPwd', '')
            default_host_id = SnmpTrapCfg.get('HostID')
            if 'Community' not in SnmpTrapCfg.keys():
                versionFlag = True
        else:
            snmpinfo.Message([res.get('data')])
            RestFunc.logout(client)
            return snmpinfo
        # 2c for T6
        version_dict = {1: "V1", 2: "V2", 3: "V3", "V2": "V2C", 0: "Disable"}
        evnent_severity = {
            'all': 'Info',
            'warning': 'Warning',
            'critical': 'Critical'
        }
        if args.version is None:
            version = str(default_trap_version)
        else:
            version = version_dict[args.version]
            editFlag = True

        if version == 'Disable':
            eventSeverity = default_enent_severity
            community = ""
            hostid = default_host_id
            v3username = ""
            authProtocol = None
            authPcode = ""
            privacy = None
            privPcode = ""
            engineId = ""
        else:
            if version == 'V3':
                if args.community is not None:
                    snmpinfo.State("Failure")
                    snmpinfo.Message(['community will be ignored in v3 trap'])
                    RestFunc.logout(client)
                    return snmpinfo
                community = default_community
                if args.v3username is None:
                    v3username = default_v3username
                else:
                    v3username = args.v3username
                    editFlag = True
                if args.authProtocol is None:
                    authProtocol = default_auth
                elif args.authProtocol == 'NONE':
                    authProtocol = 'NONE'
                    editFlag = True
                else:
                    authProtocol = args.authProtocol
                    editFlag = True
                if authProtocol == "":
                    authProtocol = "NONE"
                if args.privProtocol is None:
                    privacy = default_priv
                elif args.privProtocol == 'NONE':
                    privacy = 'NONE'
                    editFlag = True
                else:
                    privacy = args.privProtocol
                    editFlag = True
                if privacy == "":
                    privacy = "NONE"
                if authProtocol == 'SHA' or authProtocol == 'MD5':
                    if args.authPassword is None:
                        if versionFlag:
                            snmpinfo.State("Failure")
                            snmpinfo.Message(
                                ['authentication password connot be empty,when authentication protocol exists'])
                            RestFunc.logout(client)
                            return snmpinfo
                        else:
                            authPcode = default_auth_pass
                    else:
                        authPcode = args.authPassword
                        editFlag = True
                        if not RegularCheckUtil.checkPass(authPcode):
                            snmpinfo.State("Failure")
                            snmpinfo.Message(['password is a string of 8 to 16 alpha-numeric characters'])
                            RestFunc.logout(client)
                            return snmpinfo
                        # authPassword = Encrypt('secret', authPassword)
                else:
                    if args.authPassword is not None:
                        snmpinfo.State("Failure")
                        snmpinfo.Message(['authentication password will be ignored with no authentication protocol'])
                        RestFunc.logout(client)
                        return snmpinfo
                    authPcode = default_auth_pass
                if privacy == 'AES' or privacy == 'DES':
                    if args.privPassword is None:
                        if versionFlag:
                            snmpinfo.State("Failure")
                            snmpinfo.Message(['privacy password connot be empty,when privacy protocol exists'])
                            RestFunc.logout(client)
                            return snmpinfo
                        else:
                            privPcode = default_priv_pass
                    else:
                        privPcode = args.privPassword
                        editFlag = True
                        if not RegularCheckUtil.checkPass(privPcode):
                            snmpinfo.State("Failure")
                            snmpinfo.Message([' password is a string of 8 to 16 alpha-numeric characters'])
                            RestFunc.logout(client)
                            return snmpinfo
                        # privPassword = Encrypt('secret', privPassword)
                else:
                    if args.privPassword is not None:
                        snmpinfo.State("Failure")
                        snmpinfo.Message(['privacy password will be ignored with no privacy protocol'])
                        RestFunc.logout(client)
                        return snmpinfo
                    privPcode = default_priv_pass

                if args.engineId is None:
                    engineId = default_engine_Id
                else:
                    engineId = args.engineId
                    editFlag = True
                    if not RegularCheckUtil.checkEngineId(engineId):
                        snmpinfo.State("Failure")
                        snmpinfo.Message(['Engine ID is a string of 10 to 48 hex characters, must even, can set NULL.'])
                        RestFunc.logout(client)
                        return snmpinfo
            elif version == 'V1' or version == 'V2C' or version == 'V2':
                if args.community is None:
                    snmpinfo.State("Failure")
                    snmpinfo.Message(['community connot be empty in v1/v2c trap.'])
                    RestFunc.logout(client)
                    return snmpinfo
                else:
                    community = args.community
                    editFlag = True
                if args.v3username is not None:
                    snmpinfo.State("Failure")
                    snmpinfo.Message(['username will be ignored in v1/v2c trap'])
                    RestFunc.logout(client)
                    return snmpinfo
                v3username = default_v3username
                if args.authProtocol is not None:
                    snmpinfo.State("Failure")
                    snmpinfo.Message(['authentication will be ignored in v1/v2c trap'])
                    RestFunc.logout(client)
                    return snmpinfo
                authProtocol = default_auth
                if args.authPassword is not None:
                    snmpinfo.State("Failure")
                    snmpinfo.Message(['authentication password will be ignored in v1/v2c trap'])
                    RestFunc.logout(client)
                    return snmpinfo
                authPcode = default_auth_pass
                if args.privProtocol is not None:
                    snmpinfo.State("Failure")
                    snmpinfo.Message(['aprivacy will be ignored in v1/v2c trap'])
                    RestFunc.logout(client)
                    return snmpinfo
                privacy = default_priv
                if args.privPassword is not None:
                    snmpinfo.State("Failure")
                    snmpinfo.Message(['privacy password will be ignored in v1/v2c trap'])
                    RestFunc.logout(client)
                    return snmpinfo
                privPcode = default_priv_pass
                if args.engineId is not None:
                    snmpinfo.State("Failure")
                    snmpinfo.Message(['engine Id will be ignored in v1/v2c trap'])
                    RestFunc.logout(client)
                    return snmpinfo
                engineId = default_engine_Id
            if args.eventSeverity is None:
                eventSeverity = default_enent_severity
            else:
                eventSeverity = evnent_severity.get(args.eventSeverity, 'Info')
                editFlag = True
            if args.hostid is None:
                hostid = default_host_id
            else:
                hostid = args.hostid
                editFlag = True

        trapinfo = {}
        SnmpTrapCfg["TrapVersion"] = version
        SnmpTrapCfg["EventLevelLimit"] = eventSeverity
        SnmpTrapCfg["Community"] = community
        SnmpTrapCfg["HostID"] = hostid
        SnmpTrapCfg["UserName"] = v3username
        SnmpTrapCfg["AUTHProtocol"] = authProtocol
        SnmpTrapCfg["AUTHPwd"] = authPcode
        SnmpTrapCfg["PrivProtocol"] = privacy
        SnmpTrapCfg["PRIVPwd"] = privPcode
        SnmpTrapCfg["EngineID"] = engineId
        SnmpTrapCfg["DeviceType"] = 255
        trapinfo["SnmpTrapCfg"] = SnmpTrapCfg

        # if not change
        if not editFlag:
            snmpinfo.State("Success")
            snmpinfo.Message(["nothing to change."])
            RestFunc.logout(client)
            return snmpinfo

        # trapinfo["CfgType"] = 0
        trapinfo["CfgType"] = "Version"
        res = RestFunc.setTrapComByRest(client, trapinfo)
        if res == {}:
            snmpinfo.State("Failure")
            snmpinfo.Message(["set snmp error"])
        elif res.get('code') == 0 and res.get('data') is not None:
            snmpinfo.State("Success")
            snmpinfo.Message([])
        elif res.get('code') != 0 and res.get('data') is not None:
            snmpinfo.State("Failure")
            snmpinfo.Message([res.get('data')])
        else:
            snmpinfo.State("Failure")
            snmpinfo.Message(["set snmp error, error code " + str(res.get('code'))])
        RestFunc.logout(client)
        return snmpinfo
